@@ -27,8 +27,8 @@ echo $1
 if [ -z $2 ];
 then
   echo "No directory or python executable given for installation !";
-  echo "Usage: install-pynx-conda-venv.sh ENVNAME PYNX_VERSION"
-  echo "   with: DIRNAME the name of the new directory to create the python virtual environement, e.g. pynx-env"
+  echo "Usage: install-pynx-conda-venv.sh ENVNAME PYTHON_VERSION PYNX_VERSION"
+  echo "   with: ENVNAME the name of the python virtual environement, e.g. pynx-env"
   echo "         PYTHON_VERSION the python version, e.g. 3.7"
   echo "         PYNX_VERSION (optional) the git tag for the pynx version to be installed"
   echo "example: install-pynx-conda-venv.sh pynx-env 3.7"
@@ -70,8 +70,12 @@ fi
 conda config --env --set channel_priority strict
 
 # Add packages from conda-forge
-conda install --yes -n $1 numpy cython scipy matplotlib fabio ipympl pyopencl ocl-icd-system pytools scikit-image silx \
-      ipython notebook h5py hdf5plugin ipywidgets cmake clfft
+if [[ "$OSTYPE" != "darwin"* ]]; then
+  conda install --yes -n $1 ocl-icd-system
+fi
+
+conda install --yes -n $1 numpy cython scipy matplotlib fabio ipympl pyopencl pytools scikit-image silx \
+      ipython notebook h5py hdf5plugin ipywidgets cmake clfft psutil
 if [ $? -ne 0 ];
 then
   echo "Conda environment scientific packages installation failed."
@@ -172,3 +176,12 @@ echo
 echo "To use the PyNX environment, use 'conda activate $1'"
 echo
 echo "To test pynx installation, run 'pynx-test.py"
+
+
+# Notes to also install paraview and ipyparaview:
+#  conda install --yes -n $1 paraview nodejs jupyterlab
+#  cd $BASEDIR/dev
+#  git clone https://github.com/NVIDIA/ipyparaview.git
+#  cd ipyparaview
+#  ./build.sh
+#  jupyter nbextension enable ipyparaview --py --sys-prefix
