@@ -8,6 +8,7 @@
 __all__ = ['CDI', 'OperatorCDI', 'save_cdi_data_cxi']
 
 import time
+import timeit
 import sys
 import warnings
 from copy import deepcopy
@@ -15,7 +16,9 @@ import numpy as np
 from scipy.fftpack import fftn, ifftn, fftshift, fftfreq
 from scipy.signal import convolve
 from ..operator import Operator, OperatorException, has_attr_not_none
-from ..version import __version__
+from ..version import get_git_version
+
+_pynx_version = get_git_version()
 from ..utils.history import History
 
 try:
@@ -39,9 +42,14 @@ class CDI:
         :param iobs: 2D/3D observed diffraction data (intensity).
           Assumed to be corrected and following Poisson statistics, will be converted to float32.
           Dimensions should be divisible by 4 and have a prime factor decomposition up to 7.
-          The following special values are allowed:
-            * values<=-1e30 are considered masked
-            * -1e30 < values < 0 are observed but used as free pixels
+          Internally, the following special values are used:
+          * values<=-1e19 are masked. Among those, values in ]-1e38;-1e19] are estimated values,
+            stored as -(iobs_est+1)*1e19, which can be used to make a loose amplitude projection.
+            Values <=-1e38 are masked (no amplitude projection applied), just below the minimum
+            float32 value
+          * -1e19 < values <= 1 are observed but used as free pixels
+          If the mask is not supplied, then it is assumed that the above special values are
+          used.
         :param support: initial support in real space (1 = inside support, 0 = outside)
         :param obj: initial object. If None, will be initialized to a complex object, with uniform random
           0<modulus<1 and 0<phase<2pi. The data is assumed to be centered in (0,0) to avoid fft shifts.
@@ -53,7 +61,7 @@ class CDI:
         self.iobs = iobs.astype(np.float32)
         # Iobs sum, taking into account masked and free pixels (stored as -iobs-1)
         self.iobs_sum = (self.iobs * (self.iobs >= 0)).sum()
-        self.iobs_sum -= self.iobs[np.logical_and(self.iobs > -1e30, self.iobs < 0)] + 1
+        self.iobs_sum -= (self.iobs[np.logical_and(self.iobs > -1e19, self.iobs < 0)] + 1).sum()
 
         if support is not None:
             self._support = support.copy().astype(np.int8)
@@ -73,8 +81,8 @@ class CDI:
         self._is_in_object_space = True
 
         if mask is not None:
-            # Avoid needing to store/transfer a mask in GPU. Flag masked pixels with iobs < -1e30
-            self.iobs[mask > 0] = -1e31
+            # Avoid needing to store/transfer a mask in GPU. Flag masked pixels with iobs < -1e38
+            self.iobs[mask > 0] = -1.05e38
 
         # reciprocal space kernel for the point spread function. This is used for partial coherence correction.
         self._k_psf = None
@@ -105,10 +113,10 @@ class CDI:
         if mask is not None:
             self.nb_observed_points = (mask == 0).sum()
         else:
-            self.nb_observed_points = self.iobs.size
+            self.nb_observed_points = (self.iobs > -1e19).sum()
 
         # Normally free pixels will be initialised later, but if this is a copy they may already be flagged
-        self.nb_free_points = ((self.iobs > -1e30) * (self.iobs < 0)).sum()
+        self.nb_free_points = ((self.iobs > -1e19) * (self.iobs < 0)).sum()
 
         self.llk_support = None  # Current value of the support log-likelihood (support regularization)
         self.llk_support_reg_fac = None  # Regularization factor for the support log-likelihood
@@ -132,6 +140,9 @@ class CDI:
 
         # History record
         self.history = History()
+        # This is used to report cycle timings
+        self.history_llk_t0 = None
+        self.history_llk_t0_cycle = None
 
     def get_x_y_z(self):
         """
@@ -166,11 +177,11 @@ class CDI:
                                  new CDI object does not have free pixels initialised.
         :return: a copy of the object.
         """
-        iobs = self.iobs
-        mask = self.iobs < -1e30
+        iobs = self.iobs.copy()
+        mask = self.iobs <= -1e19
 
         if copy_free_pixels is False:
-            tmp = np.logical_and(self.iobs > -1e30, self.iobs < 0)
+            tmp = np.logical_and(self.iobs > -1e19, self.iobs < 0)
             iobs[tmp] = -(self.iobs[tmp] + 1)
 
         cdi = CDI(iobs, support=self._support, obj=self._obj, mask=mask,
@@ -321,7 +332,7 @@ class CDI:
 
     def set_mask(self, mask, shift=False):
         """
-        Set the mask data array. Note that since the mask is stored by setting observed intensitied of masked
+        Set the mask data array. Note that since the mask is stored by setting observed intensities of masked
         pixels to negative values, it is not possible to un-mask pixels.
 
         :param obj: the 2D or 3D CDI mask array
@@ -334,7 +345,7 @@ class CDI:
         if shift:
             mask = fftshift(mask).astype(np.int8)
         iobs = self.get_iobs()
-        iobs[mask > 0] = -1e31
+        iobs[mask > 0] = -1.05e38
         self.set_iobs(iobs)
 
     def init_free_pixels(self, ratio=5e-2, island_radius=3, exclude_zone_center=0.05, verbose=False):
@@ -347,7 +358,7 @@ class CDI:
         :return: nothing. Free pixel values are modified as iobs_free = -iobs - 1
         """
         # Clear previous free mask
-        tmp = np.logical_and(self.iobs > -1e30, self.iobs < 0)
+        tmp = np.logical_and(self.iobs > -1e19, self.iobs < 0)
         self.iobs[tmp] = -(self.iobs[tmp] + 1)
 
         if True:
@@ -396,7 +407,7 @@ class CDI:
             m = (convolve(m, k, 'same') > 0.5).astype(np.int8)
 
         # Exclude masked pixels
-        m[self.iobs < -1e30] = False
+        m[self.iobs <= -1e19] = False
 
         # Exclude zone-center
         if m.ndim == 2:
@@ -424,13 +435,13 @@ class CDI:
         Set the free pixel mask. Assumes the free pixel mask is correct (excluding center, bad pixels)
         :param m: the boolean mask (1=masked, 0 otherwise)
         :param shift: if True, the data array will be fft-shifted so that the center of the stored data is
-                      in the corner of the array. [default: the array is already shifted]
-        :return:
+        in the corner of the array. [default: the array is already shifted]
+        :return: nothing
         """
         if shift:
             m = fftshift(m)
         # Clear previous free mask
-        tmp = np.logical_and(self.iobs > -1e30, self.iobs < 0)
+        tmp = np.logical_and(self.iobs > -1e38, self.iobs < 0)
         self.iobs[tmp] = -(self.iobs[tmp] + 1)
 
         # Flag free pixels
@@ -444,7 +455,7 @@ class CDI:
         self.set_iobs(self.iobs)  # This will set the timestamp
 
     def get_free_pixel_mask(self):
-        return np.logical_and(self.iobs > -1e30, self.iobs < 0).astype(np.bool)
+        return np.logical_and(self.iobs > -1e19, self.iobs < -0.5).astype(np.bool)
 
     def save_data_cxi(self, filename, sample_name=None, experiment_id=None, instrument=None, process_parameters=None):
         """
@@ -458,13 +469,13 @@ class CDI:
         """
         # Remove mask
         iobs = self.iobs.copy()
-        mask = self.iobs < -1e30
+        mask = self.iobs <= -1e19
         iobs[mask] = 0
         # Remove free mask
         tmp = iobs < 0
         iobs[tmp] = -(iobs[tmp] + 1)
 
-        save_cdi_data_cxi(filename, iobs=self.iobs, wavelength=self.wavelength,
+        save_cdi_data_cxi(filename, iobs=iobs, wavelength=self.wavelength,
                           detector_distance=self.detector_distance,
                           pixel_size_detector=self.pixel_size_detector, mask=mask, sample_name=sample_name,
                           experiment_id=experiment_id, instrument=instrument, iobs_is_fft_shifted=True,
@@ -528,7 +539,7 @@ class CDI:
         entry.attrs['NX_class'] = 'NXentry'
         # entry.create_dataset('title', data='1-D scan of I00 v. mr')
         entry.attrs['default'] = 'data_1'
-        entry.create_dataset("program_name", data="PyNX %s" % (__version__))
+        entry.create_dataset("program_name", data="PyNX %s" % _pynx_version)
         entry.create_dataset("start_time", data=time.strftime("%Y-%m-%dT%H:%M:%S%z", time.localtime(time.time())))
         if experiment_id is not None:
             entry.create_dataset("experiment_id", data=experiment_id)
@@ -656,7 +667,7 @@ class CDI:
         process_1 = image_1.create_group("process_1")
         process_1.attrs['NX_class'] = 'NXprocess'
         process_1.create_dataset("program", data='PyNX')  # NeXus spec
-        process_1.create_dataset("version", data="%s" % __version__)  # NeXus spec
+        process_1.create_dataset("version", data="%s" % _pynx_version)  # NeXus spec
         process_1.create_dataset("command", data=command)  # CXI spec
 
         if process_notes is not None:  # Notes
@@ -787,13 +798,17 @@ class CDI:
         if mode == 'llk':
             llk = self.get_llk()
             algo = ''
-            dt = 0
+            if self.history_llk_t0_cycle is not None:
+                dt = (timeit.default_timer() - self.history_llk_t0) / (self.cycle - self.history_llk_t0_cycle)
+                kwargs['dt'] = dt
+            else:
+                dt = 0
+            self.history_llk_t0 = timeit.default_timer()
+            self.history_llk_t0_cycle = self.cycle
             if 'algorithm' in kwargs:
                 algo = kwargs['algorithm']
-            if 'dt' in kwargs:
-                dt = kwargs['dt']
             if verbose:
-                print("%4s #%3d LLK= %8.2f[%8.3f](p) %8.3f[%8.3f](g) %8.3f[%8.3f](e), nb photons=%e, "
+                print("%4s #%3d LLK= %8.3f[%8.3f](p) %8.3f[%8.3f](g) %8.3f[%8.3f](e), nb photons=%e, "
                       "support:nb=%6d (%6.3f%%) average=%10.2f max=%10.2f, dt/cycle=%6.4fs" % (
                           algo, self.cycle, llk[0], llk[3], llk[1], llk[4], llk[2], llk[5], self.nb_photons_calc,
                           self.nb_point_support, self.nb_point_support / self._obj.size * 100,
@@ -877,7 +892,7 @@ def save_cdi_data_cxi(filename, iobs, wavelength=None, detector_distance=None, p
     f.create_dataset("cxi_version", data=140)
 
     entry_1 = f.create_group("entry_1")
-    entry_1.create_dataset("program_name", data="PyNX %s" % (__version__))
+    entry_1.create_dataset("program_name", data="PyNX %s" % _pynx_version)
     entry_1.create_dataset("start_time", data=time.strftime("%Y-%m-%dT%H:%M:%S%z", time.localtime(time.time())))
     entry_1.attrs['NX_class'] = 'NXentry'
     # entry_1.create_dataset('title', data='1-D scan of I00 v. mr')
@@ -964,7 +979,7 @@ def save_cdi_data_cxi(filename, iobs, wavelength=None, detector_distance=None, p
     process_1 = data_1.create_group("process_1")
     process_1.attrs['NX_class'] = 'NXprocess'
     process_1.create_dataset("program", data='PyNX')  # NeXus spec
-    process_1.create_dataset("version", data="%s" % (__version__))  # NeXus spec
+    process_1.create_dataset("version", data="%s" % _pynx_version)  # NeXus spec
     process_1.create_dataset("command", data=command)  # CXI spec
 
     # Configuration & results of process: custom ESRF data policy
