@@ -20,6 +20,7 @@ from scipy.ndimage import gaussian_filter
 from scipy.ndimage.measurements import center_of_mass
 
 from pynx.utils.matplotlib import pyplot as plt
+from matplotlib.colors import LogNorm
 
 from ..utils.plot_utils import complex2rgbalin, complex2rgbalog, insertColorwheel, cm_phase
 from ..utils.math import llk_euclidian, llk_gaussian, llk_poisson
@@ -107,7 +108,7 @@ class CPUOperatorCDI(OperatorCDI):
             src = obj._obj
         else:
             src = obj._cpu_obj_view[i_source]
-        if i_dest is 0:
+        if i_dest == 0:
             obj._cpu_obj = np.empty_like(src)
             dest = obj._obj
         else:
@@ -273,8 +274,8 @@ class ImshowRGBA(OperatorCDI):
         plt.title("Object")
         insertColorwheel(left=.02, bottom=.0, width=.1, height=.1, text_col='black', fs=10)
         try:
-            # Not supported on all backends (e.g. nbagg)
             plt.draw()
+            plt.gcf().canvas.draw()
             plt.pause(.001)
         except:
             pass
@@ -293,10 +294,11 @@ class ShowCDI(OperatorCDI):
         NB: this is a CPU version, which will not display the calculated intensity
     """
 
-    def __init__(self, fig_num=None, i=None):
+    def __init__(self, fig_num=-1, i=None):
         """
 
         :param fig_num: the matplotlib figure number. if None, a new figure will be created each time.
+            If -1 (the default), the current figure will be re-used.
         :param i: if the object is 3D, display the ith plane (default: the center one)
         """
         super(ShowCDI, self).__init__()
@@ -321,12 +323,16 @@ class ShowCDI(OperatorCDI):
             icalc = self.get_icalc(cdi)
             support = cdi.get_support()
 
-        tmp = np.logical_and(iobs > -1e30, iobs < 0)
+        tmp = np.logical_and(iobs > -1e19, iobs < 0)
         if tmp.sum() > 0:
             # change back free pixels to their real intensity
             iobs[tmp] = -iobs[tmp] - 1
+        iobs[iobs < 0] = 0
 
-        plt.figure(self.fig_num)
+        if self.fig_num != -1:
+            plt.figure(self.fig_num)
+        else:
+            plt.gcf()
         plt.clf()
         plt.subplot(221)
         if support is not None:
@@ -356,17 +362,18 @@ class ShowCDI(OperatorCDI):
             plt.imshow(fftshift(np.angle(obj)), cmap=cm_phase)
         plt.title('Object Phase')
 
-        mi, ma = np.log10(max(iobs.min(), 0.5)), np.log10(iobs.max())
+        mi, ma = max(iobs.min(), 0.5), iobs.max()
         plt.subplot(223)
         if icalc is not None:
-            plt.imshow(np.log10(fftshift(icalc)), vmin=mi, vmax=ma, origin='lower')
+            plt.imshow(fftshift(icalc), norm=LogNorm(vmin=mi, vmax=ma), origin='lower')
         plt.title('Calculated intensity')
 
         plt.subplot(224)
-        plt.imshow(np.log10(fftshift(iobs)), vmin=mi, vmax=ma, origin='lower')
+        plt.imshow(fftshift(iobs), norm=LogNorm(vmin=mi, vmax=ma), origin='lower')
         plt.title('Observed intensity')
         try:
             plt.draw()
+            plt.gcf().canvas.draw()
             plt.pause(.002)
         except:
             pass
@@ -588,7 +595,7 @@ class ApplyAmplitude(CPUOperatorCDI):
         """
 
         :param calc_llk: if true, the log-likelihood will be calculated and stored in the object
-        :param zero_mask: if True, masked pixels (iobs<-1e30) are forced to zero, otherwise the calculated
+        :param zero_mask: if True, masked pixels (iobs<-1e19) are forced to zero, otherwise the calculated
                           complex amplitude is kept with an optional scale factor.
         :param scale_in: a scale factor by which the input values should be multiplied, typically because of FFT
         :param scale_out: a scale factor by which the output values should be multiplied, typically because of FFT
@@ -663,7 +670,7 @@ class ER(CPUOperatorCDI):
         :param show_cdi: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object intensity, as for ShowCDI()
-        :param zero_mask: if True, masked pixels (iobs<-1e30) are forced to zero, otherwise the calculated
+        :param zero_mask: if True, masked pixels (iobs<-1e19) are forced to zero, otherwise the calculated
                           complex amplitude is kept with an optional scale factor.
         """
         super(ER, self).__init__()
@@ -746,7 +753,7 @@ class CF(CPUOperatorCDI):
         :param show_cdi: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object intensity, as for ShowCDI()
-        :param zero_mask: if True, masked pixels (iobs<-1e30) are forced to zero, otherwise the calculated
+        :param zero_mask: if True, masked pixels (iobs<-1e19) are forced to zero, otherwise the calculated
                           complex amplitude is kept with an optional scale factor.
         """
         super(CF, self).__init__()
@@ -832,7 +839,7 @@ class HIO(CPUOperatorCDI):
         :param show_cdi: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object intensity, as for ShowCDI()
-        :param zero_mask: if True, masked pixels (iobs<-1e30) are forced to zero, otherwise the calculated
+        :param zero_mask: if True, masked pixels (iobs<-1e19) are forced to zero, otherwise the calculated
                           complex amplitude is kept with an optional scale factor.
         """
         super(HIO, self).__init__()
@@ -919,7 +926,7 @@ class RAAR(CPUOperatorCDI):
         :param show_cdi: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object intensity, as for ShowCDI()
-        :param zero_mask: if True, masked pixels (iobs<-1e30) are forced to zero, otherwise the calculated
+        :param zero_mask: if True, masked pixels (iobs<-1e19) are forced to zero, otherwise the calculated
                           complex amplitude is kept with an optional scale factor.
         """
         super(RAAR, self).__init__()
@@ -992,7 +999,7 @@ class GPS(CPUOperatorCDI):
         :param show_cdi: if a positive integer number N, the object & probe will be displayed every N cycle.
                                By default 0 (no plot)
         :param fig_num: the number of the figure to plot the object intensity, as for ShowCDI()
-        :param zero_mask: if True, masked pixels (iobs<-1e30) are forced to zero, otherwise the calculated
+        :param zero_mask: if True, masked pixels (iobs<-1e19) are forced to zero, otherwise the calculated
                           complex amplitude is kept with an optional scale factor.
         """
         super(GPS, self).__init__()
@@ -1074,7 +1081,7 @@ class GPS(CPUOperatorCDI):
             cdi = ApplyAmplitude(calc_llk=calc_llk, zero_mask=self.zero_mask) * cdi
 
             # obj = z_k+1 = (1 - epsilon) * sqrt(iobs) * exp(i * arg(^z)) + epsilon * z_k
-            cdi._obj = (1-epsilon) * cdi._obj + epsilon * z
+            cdi._obj = (1 - epsilon) * cdi._obj + epsilon * z
 
             if calc_llk:
                 # Average time/cycle over the last N cycles
