@@ -21,6 +21,15 @@ import shutil
 from pynx.cdi.test.test_cdi import make_cdi_data, make_cdi_data_file
 from pynx.processing_unit import has_opencl, has_cuda
 
+has_mpi = False
+try:
+    from mpi4py import MPI
+
+    if shutil.which('mpiexec') is not None:
+        has_mpi = True
+except ImportError:
+    pass
+
 exclude_cuda = False
 exclude_opencl = False
 if 'PYNX_PU' in os.environ:
@@ -68,7 +77,7 @@ class TestCDIRunner(unittest.TestCase):
         # Test suquentially several options. Could also try combined options..
         for options in [[], ["roi=16,112,16,112,16,112"], ["positivity"], ["detwin=0"], ["output_format=npz"],
                         ["support_size=50"], ["support_only_shrink"], ["support_update_border_n=3"],
-                        ["rebin=2"], ["rebin=2,1,2"]]:
+                        ["rebin=2"], ["rebin=2,1,2"], ["mask_interp=16,2"]]:
             with subprocess.Popen(args + options, stderr=subprocess.PIPE,
                                   stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
                 stdout, stderr = p.communicate(timeout=200)
@@ -102,7 +111,7 @@ class TestCDIRunner(unittest.TestCase):
         # Test suquentially several options. Could also try combined options..
         for options in [[], ["roi=16,112,16,112,16,112"], ["positivity"], ["detwin=0"], ["output_format=npz"],
                         ["support_size=50"], ["support_only_shrink"], ["support_update_border_n=3"],
-                        ["rebin=2"], ["rebin=2,1,2"]]:
+                        ["rebin=2"], ["rebin=2,1,2"], ["mask_interp=16,2"]]:
             with subprocess.Popen(args + options, stderr=subprocess.PIPE,
                                   stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
                 stdout, stderr = p.communicate(timeout=200)
@@ -136,7 +145,7 @@ class TestCDIRunner(unittest.TestCase):
         # Test sequentially several options. Could also try combined options..
         for options in [[], ["roi=16,112,16,112"], ["positivity"], ["detwin=0"], ["output_format=npz"],
                         ["support_size=50"], ["support_only_shrink"], ["support_update_border_n=3"],
-                        ["rebin=2"], ["rebin=2,1"]]:
+                        ["rebin=2"], ["rebin=2,1"], ["mask_interp=16,2"]]:
             with subprocess.Popen(args + options, stderr=subprocess.PIPE,
                                   stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
                 stdout, stderr = p.communicate(timeout=200)
@@ -170,7 +179,7 @@ class TestCDIRunner(unittest.TestCase):
         # Test suquentially several options. Could also try combined options..
         for options in [[], ["roi=16,112,16,112"], ["positivity"], ["detwin=0"], ["output_format=npz"],
                         ["support_size=50"], ["support_only_shrink"], ["support_update_border_n=3"],
-                        ["rebin=2"], ["rebin=2,1"]]:
+                        ["rebin=2"], ["rebin=2,1"], ["nb_run=3", "nb_run_keep=1"], ["mask_interp=16,2"]]:
             with subprocess.Popen(args + options, stderr=subprocess.PIPE,
                                   stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
                 stdout, stderr = p.communicate(timeout=200)
@@ -195,6 +204,46 @@ class TestCDIRunner(unittest.TestCase):
             res = p.returncode
             self.assertFalse(res, msg=stderr.decode())
 
+    @unittest.skipIf(has_mpi is False, 'no MPI support')
+    def test_cdi_runner_id01_2d_cxi_mpi_run(self):
+        my_env = os.environ.copy()
+        path = make_cdi_data_file(shape=(128, 128), file_type='cxi', dir=self.tmp_dir)
+        args = ['mpiexec', '-n', '2', 'pynx-id01cdi.py', 'data=%s' % path, "nb_raar=50",
+                "nb_hio=50", "nb_er=50", "support_update_period=20", "mpi=run",
+                "nb_run=4", "nb_run_keep=2"]
+        # Test several options. Could also try combined options..
+        for options in [[]]:
+            with subprocess.Popen(args + options, stderr=subprocess.PIPE,
+                                  stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
+                stdout, stderr = p.communicate(timeout=200)
+                res = p.returncode
+                msg = "Failed command-line:"
+                for a in p.args:
+                    msg += " " + a
+                msg += "\n"
+                self.assertFalse(res, msg=msg + stderr.decode())
+
+    @unittest.skipIf(has_mpi is False, 'no MPI support')
+    def test_cdi_runner_id01_2d_cxi_mpi_scan(self):
+        my_env = os.environ.copy()
+        path = make_cdi_data_file(shape=(128, 128), file_type='cxi', dir=self.tmp_dir)
+        for i in range(2):
+            os.system("ln -sf %s %s/scan%02d.cxi" % (path, self.tmp_dir, i))
+        path = self.tmp_dir + "/scan%02d.cxi"
+        args = ['mpiexec', '-n', '2', 'pynx-id01cdi.py', 'data=' + path, "nb_raar=50",
+                "nb_hio=50", "nb_er=50", "support_update_period=20", "mpi=scan", "scan=0,1"]
+        # Test several options. Could also try combined options..
+        for options in [[]]:
+            with subprocess.Popen(args + options, stderr=subprocess.PIPE,
+                                  stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
+                stdout, stderr = p.communicate(timeout=200)
+                res = p.returncode
+                msg = "Failed command-line:"
+                for a in p.args:
+                    msg += " " + a
+                msg += "\n"
+                self.assertFalse(res, msg=msg + stderr.decode())
+
 
 def suite():
     loadTests = unittest.defaultTestLoader.loadTestsFromTestCase
