@@ -91,7 +91,8 @@ def make_cdi_data(shape=(128, 128, 128), obj_shape='rectangle', nb_photons=1e9):
     return obj, d
 
 
-def make_cdi_data_file(shape=(128, 128, 128), obj_shape='rectangle', file_type='cxi', nb_photons=1e9, dir=None):
+def make_cdi_data_file(shape=(128, 128, 128), obj_shape='rectangle', file_type='cxi', nb_photons=1e9, dir=None,
+                       mask_fraction=0.02):
     """
     Create a CDI data file.
     :param shape: the shape of the data file, either 2D or 3D.
@@ -100,13 +101,17 @@ def make_cdi_data_file(shape=(128, 128, 128), obj_shape='rectangle', file_type='
     :param file_type: either npz or cxi
     :param nb_photons: the total number of photons in the data array
     :param dir: the directory where the file will be created
+    :param mask_fraction: fraction of masked pixels (for CXI only)
     :return: the file name
     """
     obj, d = make_cdi_data(shape=shape, obj_shape=obj_shape, nb_photons=nb_photons)
+    # Mask 2% of pixels
+    mask = np.zeros(d.shape, dtype=np.bool)
+    mask[np.random.uniform(0, 1, mask.shape) < mask_fraction] = True
 
     if file_type == 'cxi':
         f, path = tempfile.mkstemp(suffix='.cxi', dir=dir)
-        save_cdi_data_cxi(path, d, wavelength=1.5e-10, detector_distance=1, pixel_size_detector=55e-6, mask=None,
+        save_cdi_data_cxi(path, d, wavelength=1.5e-10, detector_distance=1, pixel_size_detector=55e-6, mask=mask,
                           sample_name=None, experiment_id=None, instrument=None, note=None, iobs_is_fft_shifted=False)
     else:
         # npz
@@ -140,9 +145,12 @@ class TestCDI(unittest.TestCase):
         :return: Nothing
         """
         if self.cdi_2d is None:
-            obj2d, d2d = make_cdi_data(obj_shape=(256, 256))
+            obj2d, d2d = make_cdi_data(shape=(256, 256))
+            # Mask 2% of pixels
+            mask = np.zeros(d2d.shape, dtype=np.bool)
+            mask[np.random.uniform(0, 1, mask.shape) < 0.02] = True
             self.cdi_2d = CDI(obj=None, pixel_size_detector=55e-6, iobs=d2d, support=obj2d, wavelength=1.5e-10,
-                              detector_distance=1)
+                              detector_distance=1, mask=mask)
             # Create initial object using auto-correlation
             self.cdi_2d = AutoCorrelationSupport() * self.cdi_2d
             self.cdi_2d.set_obj(np.random.uniform(0, 1, d2d.shape) * self.cdi_2d.get_support())
@@ -158,7 +166,10 @@ class TestCDI(unittest.TestCase):
         :return: Nothing
         """
         if self.cdi_3d is None:
-            obj3d, d3d = make_cdi_data(obj_shape=(128, 128, 128))
+            obj3d, d3d = make_cdi_data(shape=(128, 128, 128))
+            # Mask 2% of pixels
+            mask = np.zeros(d3d.shape, dtype=np.bool)
+            mask[np.random.uniform(0, 1, mask.shape) < 0.02] = True
             self.cdi_3d = CDI(obj=None, pixel_size_detector=55e-6, iobs=d3d, support=obj3d, wavelength=1.5e-10,
                               detector_distance=1)
             # Create initial object using auto-correlation
@@ -290,8 +301,91 @@ class TestCDI(unittest.TestCase):
             # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
             cdi = op.FreePU() * cdi
 
-    def run_algorithm_consistency(self, op, mod_op1=clop, mod_op2=cuop, atol=1e-5, rtol=1e-4, rtol_llk=1e-2, nbtol=10,
-                                  ndim=2):
+    @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests excluded")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
+    def test_SupportUpdate_opencl(self):
+        self.make_cdi_obj2d()
+        self.make_cdi_obj3d()
+        op = clop
+        for cdi, obj0, support0 in [(self.cdi_2d, self.obj0_2d, self.support0_2d),
+                                    (self.cdi_3d, self.obj0_3d, self.support0_3d)]:
+            cdi.set_obj(obj0)
+            cdi.set_support(support0)
+            # Compute LLK before and after
+            cdi = op.LLK() * cdi
+            llk0 = cdi.get_llkn()
+            ra = op.RAAR(positivity=True, calc_llk=0) ** 10
+            cdi = op.SupportUpdate(force_shrink=True) * ra * cdi
+            cdi = op.SupportUpdate() * ra * cdi
+            cdi = op.SupportUpdate(update_border_n=2) * ra * cdi
+            cdi = op.SupportUpdate(method='max') * ra * cdi
+            cdi = op.LLK() * cdi
+            llk1 = cdi.get_llkn()
+            # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
+            cdi = op.FreePU() * cdi
+
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    def test_SupportUpdate_cuda(self):
+        self.make_cdi_obj2d()
+        self.make_cdi_obj3d()
+        op = cuop
+        for cdi, obj0, support0 in [(self.cdi_2d, self.obj0_2d, self.support0_2d),
+                                    (self.cdi_3d, self.obj0_3d, self.support0_3d)]:
+            cdi.set_obj(obj0)
+            cdi.set_support(support0)
+            # Compute LLK before and after
+            cdi = op.LLK() * cdi
+            llk0 = cdi.get_llkn()
+            ra = op.RAAR(positivity=True, calc_llk=0) ** 10
+            cdi = op.LLK() * op.SupportUpdate(force_shrink=True) * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate() * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate(update_border_n=2) * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate(method='max') * ra * cdi
+            llk1 = cdi.get_llkn()
+            # print("HIO**10: LLK %8.2f -> %8.2f" %(llk0, llk1))
+            cdi = op.FreePU() * cdi
+
+    def test_InterpIobsMask_opencl(self):
+        self.make_cdi_obj2d()
+        self.make_cdi_obj3d()
+        op = clop
+        for cdi, obj0, support0 in [(self.cdi_2d, self.obj0_2d, self.support0_2d),
+                                    (self.cdi_3d, self.obj0_3d, self.support0_3d)]:
+            cdi.set_obj(obj0)
+            cdi.set_support(support0)
+            # Compute LLK before and after
+            cdi = op.LLK() * cdi
+            llk0 = cdi.get_llkn()
+            ra = op.RAAR(positivity=True, calc_llk=0) ** 10
+            cdi = op.LLK() * op.SupportUpdate(force_shrink=True) * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate() * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate(update_border_n=2) * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate(method='max') * ra * cdi
+            llk1 = cdi.get_llkn()
+            cdi = op.FreePU() * cdi
+
+    def test_InterpIobsMask_cuda(self):
+        self.make_cdi_obj2d()
+        self.make_cdi_obj3d()
+        op = cuop
+        for cdi, obj0, support0 in [(self.cdi_2d, self.obj0_2d, self.support0_2d),
+                                    (self.cdi_3d, self.obj0_3d, self.support0_3d)]:
+            cdi.set_obj(obj0)
+            cdi.set_support(support0)
+            # Compute LLK before and after
+            cdi = op.LLK() * cdi
+            llk0 = cdi.get_llkn()
+            ra = op.RAAR(positivity=True, calc_llk=0) ** 10
+            cdi = op.LLK() * op.SupportUpdate(force_shrink=True) * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate() * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate(update_border_n=2) * ra * cdi
+            cdi = op.LLK() * op.SupportUpdate(method='max') * ra * cdi
+            llk1 = cdi.get_llkn()
+            cdi = op.FreePU() * cdi
+
+    def run_algorithm_consistency(self, op, mod_op1=clop, mod_op2=cuop, atol=1e-5, rtol=1e-4,
+                                  rtol_llk=1e-2, nbtol=10, nbfractol=0.001, ndim=2):
         """
         Test the consistency of the calculation using e.g. OpenCL and CUDA, for a given operator
         :param op: the operator to be used, written in a generic way such as 'op.Operator()', where 'op.' will be 
@@ -302,8 +396,8 @@ class TestCDI(unittest.TestCase):
         :param atol: absolute tolerance, relative to the maximum of the object (or FT'd object) array.
         :param rtol: relative tolerance for the object (or FT'd object) array.
         :param rtol_llk: relative tolerance for the calculated LLK.
-        :param nbtol: number of points which can be above the numerical tolerance, as well as number of points which
-                      can differ in the support array.
+        :param nbtol: number of points which can be above the numerical tolerance
+        :param nbfractol: fraction of points which can differ in the support array.
         :param ndim: number of dimensions. Either 2 or 3.
         :return: nothing
         """
@@ -348,14 +442,17 @@ class TestCDI(unittest.TestCase):
 
         atol = np.abs(objcl).max() * atol
         nb = (supcl != supcu).sum()
-        self.assertTrue((nb < nbtol),
-                        msg="%s and %s operation: '%s' must give close results (dSupport=%d)" % (pu1, pu2, op, nb))
+        nbf = nb / obj0.size * 100
+        self.assertTrue((nb < nbfractol*obj0.size),
+                        msg="%s and %s operation (%dD): '%s' must give close results (dSupport=%d [%4.2f%%])" %
+                            (pu1, pu2, ndim, op, nb, nbf))
         nb = (abs(objcu - objcl) > (rtol * abs(objcl) + atol)).sum()
         self.assertTrue(nb < nbtol,
-                        msg="%s and %s operation: '%s' must give close results (Psi, nbdiff=%d)" % (pu1, pu2, op, nb))
+                        msg="%s and %s operation (%dD): '%s' must give close results (Psi, nbdiff=%d)" %
+                            (pu1, pu2, ndim, op, nb))
         self.assertTrue((llkcl - llkcu) < (llkcl * rtol_llk + 1e-4),
-                        msg="%s and %s operation: '%s' must give close results (LLK= %8g, %8g)" % (
-                        pu1, pu2, op, llkcl, llkcu))
+                        msg="%s and %s operation (%dD): '%s' must give close results (LLK= %8g, %8g)" % (
+                            pu1, pu2, ndim, op, llkcl, llkcu))
 
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests excluded")
     @unittest.skipIf(has_cuda is False, 'no CUDA support')
@@ -363,7 +460,10 @@ class TestCDI(unittest.TestCase):
     @unittest.skipIf(has_opencl is False, 'no OpenCL support')
     def test_consistency_opencl_cuda(self):
         for ndim in (2, 3):
-            for op in ['op.FT(scale=True)', 'op.IFT(scale=True)', 'op.FourierApplyAmplitude()', 'op.SupportUpdate()',
+            for op in ['op.FT(scale=True)', 'op.IFT(scale=True)', 'op.FourierApplyAmplitude()',
+                       'op.SupportUpdate(method="rms", threshold_relative=0.3)',
+                       'op.SupportUpdate(method="max")',
+                       'op.SupportUpdate(method="average", threshold_relative=0.3)',
                        'op.ER()', 'op.ER(positivity=True)']:
                 self.run_algorithm_consistency(op, mod_op1=clop, mod_op2=cuop, ndim=ndim)
 
