@@ -11,6 +11,13 @@ from .cuda_device import has_cuda
 from .opencl_device import has_opencl
 import numpy as np
 from scipy.fftpack import fftn, ifftn, fftshift
+try:
+    from mpi4py import MPI
+    mpi = MPI.COMM_WORLD
+    if mpi.Get_size() == 1:
+        mpi = None
+except ImportError:
+    mpi = None
 
 
 class ProcessingUnitException(Exception):
@@ -201,6 +208,9 @@ class ProcessingUnit(object):
         this case, the rank always corresponds to the order of the device, and not the fft speed. Last case, the first
         part can match the gpu name, and the second part can be a rank (if using a device with multiple identical GPU).
 
+        If MPI is used and multiple devices are available, the selection is made according to the MPI rank,
+        after the gpu name filtering is taken into account.
+
         This will call set_device() with the selected device.
 
         :param gpu_name: a (sub)string matching the name of the gpu to be used. This can also be a list of
@@ -210,7 +220,8 @@ class ProcessingUnit(object):
                          they are found.
         :param ranking: default is 'fft_speed' (or 'fft'), all GPU are tested for FFT speed, and chosen from that
                         ranking. If ranking is 'order', they will be listed as they are found by CUDA/OpenCL.
-                        This is useful to select a GPU just based on its name, without the overhead of the FFT test
+                        This is useful to select a GPU just based on its name, without the overhead of the FFT test.
+                        Ignored if using MPI.
         :param language: either 'opencl', 'cuda', 'cpu', or None. If None, the preferred language is cuda>opencl>cpu
         :param verbose: True or False (default) for verbosity
         :return: True if a GPU (or CPU) could be selected. An exception is returned if no GPU was found,
@@ -232,13 +243,16 @@ class ProcessingUnit(object):
                     try:
                         gpu_rank = int(s)
                         ranking = 'order'
-                        if (verbose):
+                        if verbose:
                             print("PyNX processing unit: use #%d device from PYNX_PU environment variable" % gpu_rank)
                     except ValueError:
                         gpu_name = s
                         if verbose:
                             print("PyNX processing unit: searching '%s' GPU from PYNX_PU environment variable" % s)
 
+        if mpi is not None:
+            ranking = 'mpi'
+
         if ranking not in ['fft_speed', 'fft']:
             # This will deactivate FFT speed, just reporting found GPU devices
             fft_shape = None
@@ -265,6 +279,12 @@ class ProcessingUnit(object):
                             s += ' [gpu_name=%s]' % str(gpu_name)
                         raise Exception(s)
 
+        if len(benchmark_results) and mpi is not None:
+            # if we have more than 1 CUDA GPU available, use MPI rank to select it
+            nb = len(benchmark_results)
+            r = mpi.Get_rank()
+            return self.set_device(benchmark_results[r % nb][0], test_fft=False, verbose=verbose)
+
         test_opencl = False
         if language is None:
             test_opencl = True
@@ -296,6 +316,13 @@ class ProcessingUnit(object):
                 return self.set_device(test_fft=False, verbose=verbose)
 
         if len(benchmark_results):
+            if mpi is not None:
+                # if we have more than 1 GPU available, use MPI rank to select it
+                # This requires that a language be selected, or each device will be listed twice for cuda+opencl
+                nb = len(benchmark_results)
+                r = mpi.Get_rank()
+                return self.set_device(benchmark_results[gpu_rank][r % nb], test_fft=False, verbose=verbose)
+
             if ranking == 'fft_speed':
                 benchmark_results = sorted(benchmark_results, key=lambda t: -t[1])
             if len(benchmark_results) <= gpu_rank:
