@@ -7,13 +7,14 @@
 
 __all__ = ['Bragg2DPtychoData', 'Bragg2DPtycho', 'OperatorBragg2DPtycho']
 
+import warnings
 import numpy as np
 from ...operator import Operator, has_attr_not_none
 from ...utils.rotation import rotate, rotation_matrix
 from ...utils.history import History
 
-
-# TODO: Merge as much as possible with the 3D Bragg case (almost identical)
+warnings.warn("PyNX: you are importing the ptycho.bragg2d module, which is highly unstable (API will vary)."
+              "Do not use it unless you know the details of current developments.")
 
 
 class Bragg2DPtychoData(object):
@@ -21,7 +22,7 @@ class Bragg2DPtychoData(object):
     This may include only part of the data from a larger dataset.
     """
 
-    def __init__(self, iobs=None, positions=None, mask=None, wavelength=None, detector=None, scattering_vector=None):
+    def __init__(self, iobs=None, positions=None, mask=None, wavelength=None, detector=None, rotation=None):
         """
         Init function. The orthonormal coordinate system to be used for the sample position follows the
         NeXus/CXI/McStas convention:
@@ -32,72 +33,101 @@ class Bragg2DPtychoData(object):
         :param iobs: 3d array with (nb_frame, ny,nx) shape observed intensity (assumed to follow Poisson statistics).
                      Data is assumed to be centered on the detector, and will be fft-shifted to be centered in (0,0).
         :param positions: (x, y, z) tuple or 2d array with ptycho probe positions in meters. The coordinates
-                          must be in the laboratory reference frame given above.
+                          must be in the laboratory reference frame given above. The positions must be given
+                          relative to the center of the modeled sample - in practice they should be centered around 0.
         :param mask: 2D or 3D mask (>0 means masked pixel) for the observed data. Can be None. If 2D, it is assumed
                      that the mask is the same for all frames.
         :param wavelength: wavelength of the experiment, in meters.
-        :param detector= {rotation_axes:(('x', 0), ('y', pi/4)), 'pixel_size':55e-6, 'distance':1}:
+        :param detector: {rotation_axes:(('x', 0), ('y', pi/4)), 'pixel_size':55e-6, 'distance':1}:
                parameters for the detector as a dictionary. The rotation axes giving the detector orientation
                will be applied in order, i.e. to find the detector direction, a vector with x=y=0, z=1 is rotated
                by the axes in the given order.
-        :param scattering_vector=(sx, sy, sz): the coordinates of the central scattering vector for all frames, taking
-               into account any tilt for multi-angle Bragg projection ptychography. The scattering vector coordinates
-               units should be inverse meters (without 2pi multiplication), i.e. the norm should be 2*sin(theta)/lambda,
-               where theta is the Bragg angle. Note that only the difference between the scattering vector and the
-               average (intensity-weighted) scattering vector is used for multi-orientation back-projection.
-               [Default: None, all frames are taken with the same orientation]
+               Mandatory entries are:
+
+               * 'rotation_axes':  a tuple (or list) of tuple, and each tuple entry should be an axis rotation around
+                     either x, y or z axis, e.g. ('x', pi/6). Each rotation is applied in order. The value of the
+                     rotation can either be given as a floating-point value in radian, or can be an array of values.
+               * 'pixel_size' : detector pixel size in meters
+               * 'distance' : sample-detector distance in meters
+
+               Optional entries are:
+
+               * 'roi_dx' and 'roi_dy': these are the shift (in pixels) of the center of the cut region-of-interest,
+                 for each frame, relative to the position of the direct beam on the detector when at rest
+                 (all angles=0). These can also be arrays.
+
+               If any of the parameters are given as an array, its size must correspond to the number of observed frames
+
+        :param rotation: a tuple or list of rotations for the sample, e.g. (('x', 0), ('y', pi/4)), giving the
+                         rotation for the sample. The entries can be given exactly as for the detector
+                         'rotation_axes' entry, e.g.:
+                         rotation_axes:(('y', np.linspace(-np.deg2rad(1),np.deg2rad(1),21)))
+
+                         If several rotation axis are given, they are applied in order to set the sample in the final
+                         orientation.
+
+                         If the values are arrays, they must have the same size as the number of frames.
         """
-        # TODO: alternatively, instead of supplying one detector geometry and scattering vectors which can vary from
-        # frame to frame, rather allow some geometry parameters to be vectors to be more general ? This
-        # would require including detector crop coordinates and rotation axes...
-
-        if scattering_vector is not None:
-            hx, hy, hz = scattering_vector
-            hx0 = hx.mean()
-            hy0 = hy.mean()
-            hz0 = hz.mean()
-            if iobs is not None:
-                iobs_sum = iobs.sum(axis=(1, 2))
-                s = iobs_sum.sum()
-                if s > 1000:  # Avoid the special case with null data for simulation
-                    hx0 = (hx * iobs_sum).sum() / s
-                    hy0 = (hy * iobs_sum).sum() / s
-                    hz0 = (hz * iobs_sum).sum() / s
-            self.s0 = (hx0, hy0, hz0)  # Average scattering vector
-            self.ds = (hx - hx0, hy - hy0, hz - hz0)  # Difference with average scattering vector
+        # Handle only nx=ny for square object pixel size
+        assert iobs.shape[-2] == iobs.shape[-1]
+        self.iobs = np.fft.fftshift(iobs, axes=(-2, -1)).astype(np.float32)
+        # Total nb of photons is used for regularization
+        self.iobs_sum = self.iobs.sum()
+        if mask is not None:
+            if mask.ndim == 2:
+                mask = np.repeat(mask[np.newaxis, :, :], iobs.shape[0],
+                                 axis=0)  # tile mask to be a 3D array with (nb_frame, ny, nx) shape
+            self.mask = np.fft.fftshift(mask.astype(np.int8), axes=(-2, -1))
+            self.iobs[self.mask > 0] = -100
         else:
-            self.s0 = (0, 0, 0)
-            tmp = np.zeros(len(iobs), dtype=np.float32)
-            self.ds = tmp.copy(), tmp.copy(), tmp.copy()
-
-        if iobs is not None:
-            assert iobs.shape[-2] == iobs.shape[-1]
-            self.iobs = np.fft.fftshift(iobs, axes=(-2, -1)).astype(np.float32)
-            # Total nb of photons is used for regularization
-            self.iobs_sum = self.iobs.sum()
-            if mask is not None:
-                if mask.ndim ==2:
-                    mask = np.repeat(mask[np.newaxis, :, :], iobs.shape[0],
-                                       axis=0)  # tile mask to be a 3D array with (nb_frame, ny, nx) shape
-                self.mask = np.fft.fftshift(mask.astype(np.int8), axes=(-2, -1))
-                self.iobs[self.mask > 0] = -100
-            else:
-                self.mask = None
-        else:
-            self.iobs = None
+            self.mask = None
 
         self.wavelength = wavelength
         self.posx, self.posy, self.posz = positions
-        self.posx -= self.posx.mean()
-        self.posy -= self.posy.mean()
-        self.posz -= self.posz.mean()
+        # if center_pos:
+        #     self.posx -= self.posx.mean()
+        #     self.posy -= self.posy.mean()
+        #     self.posz -= self.posz.mean()
         self.detector = detector
+
+        # Average orientation for the detector
         self.m = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], dtype=np.float32)
         for ax, angle in detector['rotation_axes']:
-            self.m = np.dot(rotation_matrix(ax, angle), self.m)
+            if np.isscalar(angle):
+                ang = angle
+            else:
+                ang = np.array(angle).mean()
+            self.m = np.dot(rotation_matrix(ax, ang), self.m)
         self.im = np.linalg.inv(self.m)
 
-        # Calculate ds coordinate in detector reference frame
+        # TODO: take into account ROI shift (roi_dx, roi_dy)
+        # Reference (average) scattering vector from detector coordinates
+        self.s0 = np.array(rotate(self.m, 0, 1 / wavelength, 0))
+        self.s0 -= np.array([0, 1 / wavelength, 0])
+        if rotation is not None:
+            for ax, angle in rotation:
+                if np.isscalar(angle):
+                    ang = angle
+                else:
+                    ang = np.array(angle).mean()
+                self.s0 = np.array(rotate(rotation_matrix(ax, -ang), self.s0[0], self.s0[1], self.s0[2]))
+        tth = 2 * np.arcsin(np.sqrt((self.s0 ** 2).sum()) * wavelength / 2)
+        print("Average scattering vector: s=(%5.3f %5.3f %5.3f)[nm-1] (2theta=%6.2f)" %
+              (self.s0[0] * 1e-9, self.s0[1] * 1e-9, self.s0[2] * 1e-9, np.rad2deg(tth)))
+
+        # Relative shift of the scattering vector for each frame (if any)
+        nb = len(self.posx)
+        self.ds = np.zeros((3, nb))
+        sx, sy, sz = np.zeros(nb), np.ones(nb) / wavelength, np.zeros(nb)
+        for ax, angle in detector['rotation_axes']:
+            sx, sy, sz = rotate(rotation_matrix(ax, angle), sx, sy, sz)
+        sy -= 1 / wavelength
+        if rotation is not None:
+            for ax, angle in rotation:
+                sx, sy, sz = rotate(rotation_matrix(ax, -angle), sx, sy, sz)
+        self.ds = sx - self.s0[0], sy - self.s0[1], sz - self.s0[2]
+
+        # Calculate ds coordinates in the detector reference frame
         self.ds1 = rotate(self.im, self.ds[0], self.ds[1], self.ds[2])
 
     def calc_ds(self, dix, diy):
@@ -107,7 +137,7 @@ class Bragg2DPtychoData(object):
         :param diy: shift in pixels on the detector along Y (detector reference frame, top to bottom). Can be a vector.
         :return: the scattering vector coordinates (sx, sy, sz) in the laboratory reference frame
         """
-        sx0, sy0, sz0 = self.s0  # Mean scattering vector for all frames, in laboratory referennce frame
+        sx0, sy0, sz0 = self.s0  # Mean scattering vector for all frames, in laboratory reference frame
         # Try to shift by 50 pixels
         sxd = sx0 * self.detector['distance'] * self.wavelength
         syd = sy0 * self.detector['distance'] * self.wavelength
@@ -131,7 +161,6 @@ class Bragg2DPtycho(object):
 
     def __init__(self, probe=None, data=None, support=None, background=None):
         """
-        Constructor.
         :param probe: the starting estimate of the probe, as a pynx wavefront object - can be 3D if modes are used.
         :param data: the Bragg2DPtychoData object with all observed frames, ptycho positions
         :param support: the support of the object, with values ranging from 0 (outside) to 100 (inside). Intermediate
@@ -561,6 +590,7 @@ class Bragg2DPtycho(object):
     def get_xyz(self, rotation=None, domain='object'):
         """
         Get x,y,z orthonormal coordinates corresponding to the object grid.
+
         :param domain: the domain over which the xyz coordinates should be returned. It should either
                                be 'object' (the default) or the probe, the only difference being that the object
                                is extended to cover all the volume scanned by the shifted probe.
@@ -663,7 +693,7 @@ class Bragg2DPtycho(object):
                 dt = kwargs['dt']
             if verbose:
                 print("%-8s #%3d LLK= %8.2f(p) %8.2f(g) %8.2f(e), nb photons=%e, dt/cycle=%6.4fs" % (
-                          algo, self.cycle, llk[0], llk[1], llk[2], self.nb_photons_calc, dt))
+                    algo, self.cycle, llk[0], llk[1], llk[2], self.nb_photons_calc, dt))
             self.history.insert(self.cycle, llk_poisson=llk[0], llk_gaussian=llk[1],
                                 llk_euclidian=llk[2], nb_photons_calc=self.nb_photons_calc,
                                 nb_obj=len(self._obj), nb_probe=len(self._probe2d._d), **kwargs)
