@@ -39,7 +39,7 @@ command-line arguments:
     
     xyrange=-2e-6,2e-6,-5e-6,-2e-6: range where the data points will be taken into 
              account (xmin, xmax, ymin, ymax). All scan positions outside this range are ignored.
-             This must be given in orginal coordinates, in meters.
+             This must be given in original coordinates, in meters.
              [default=No spatial restrictions]
 """
 
@@ -51,10 +51,69 @@ for k, v in params_beamline.items():
 
 
 class PtychoRunnerScanCXI(PtychoRunnerScan):
-    def __init__(self, params, scan):
-        super(PtychoRunnerScanCXI, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(PtychoRunnerScanCXI, self).__init__(params, scan, timings=timings)
+
+    def load_scan(self):
+        if os.path.isfile(self.params['cxifile']) is False:
+            raise PtychoRunnerException("CXI file does not exist: %s" % (self.params['cxifile']))
+        cxi = h5py.File(self.params['cxifile'], 'r')
+        x, y = cxi['/entry_1/sample_1/geometry_1/translation'][0:2]
+
+        if len(x) < 4:
+            raise PtychoRunnerException("Less than 4 scan positions, is this a ptycho scan ?")
+
+        imgn = np.arange(len(x), dtype=np.int)
+
+        if '/entry_1/data_1/monitor' in cxi:
+            mon = cxi['/entry_1/data_1/monitor'][()]
+            mon0 = np.median(mon)
+            mon /= mon0
+            self.validframes = np.where(mon > 0.1)
+            if len(self.validframes) != len(mon):
+                print('WARNING: The following frames have a monitor value < 0.1'
+                      ' the median value and will be ignored (no beam ?)')
+                print(np.where(mon <= (mon0 * 0.1)))
+            x = np.take(x, self.validframes)
+            y = np.take(y, self.validframes)
+            imgn = np.take(imgn, self.validframes)
+        else:
+            mon = None
+
+        if 'xyrange' in self.params:
+            xmin, xmax, ymin, ymax = self.params['xyrange']
+            print("Restricting scan positions to %f < x < %f and %f < y < %f" % (xmin, xmax, ymin, ymax))
+            idx = np.where((x >= xmin) * (x <= xmax) * (y >= ymin) * (y <= ymax))[0]
+            if len(idx) < 10:
+                raise PtychoRunnerException("Only %d points remaining after applying the xyrange "
+                                            "constraint. original range: %5e<x<%5e %5e<y<%5e"
+                                            % (len(idx), x.min(), x.max(), y.min(), y.max()))
+            else:
+                print("   ... %d/%d remaining positions" % (len(idx), len(x)))
+            imgn = imgn.take(idx)
+            x = x.take(idx)
+            y = y.take(idx)
+
+        if self.params['moduloframe'] is not None:
+            n1, n2 = self.params['moduloframe']
+            idx = np.where(imgn % n1 == n2)[0]
+            imgn = imgn.take(idx)
+            x = x.take(idx)
+            y = y.take(idx)
+
+        if self.params['maxframe'] is not None:
+            N = self.params['maxframe']
+            if len(imgn) > N:
+                print("MAXFRAME: only using first %d frames" % (N))
+                imgn = imgn[:N]
+                x = x[:N]
+                y = y[:N]
+        self.x, self.y, self.imgn = x, y, imgn
 
     def load_data(self):
+        imgn = self.imgn
+        if imgn is None:
+            raise PtychoRunnerException("load_data(): imgn is None. Did you call load_scan() before ?")
         if os.path.isfile(self.params['cxifile']) is False:
             raise PtychoRunnerException("CXI file does not exist: %s" % (self.params['cxifile']))
         cxi = h5py.File(self.params['cxifile'], 'r')
@@ -63,7 +122,6 @@ class PtychoRunnerScanCXI(PtychoRunnerScan):
             self.params['nrj'] = cxi['/entry_1/instrument_1/source_1/energy'][()] / 1.60218e-16
         self.params['detectordistance'] = cxi['/entry_1/instrument_1/detector_1/distance'][()]
         self.params['pixelsize'] = cxi['/entry_1/instrument_1/detector_1/x_pixel_size'][()]
-        self.x, self.y = cxi['/entry_1/sample_1/geometry_1/translation'][0:2]
 
         if self.params['scan'] is None and '/entry_1/data_1/process_1/configuration/scan' in cxi:
             try:
@@ -72,49 +130,6 @@ class PtychoRunnerScanCXI(PtychoRunnerScan):
             except:
                 pass
 
-        if len(self.x) < 4:
-            raise PtychoRunnerException("Less than 4 scan positions, is this a ptycho scan ?")
-
-        imgn = np.arange(len(self.x), dtype=np.int)
-
-        if '/entry_1/data_1/monitor' in cxi:
-            mon = cxi['/entry_1/data_1/monitor'][()]
-            mon0 = np.median(mon)
-            mon /= mon0
-            self.validframes = np.where(mon > 0.1)
-            if len(self.validframes) != len(mon):
-                print(
-                    'WARNING: The following frames have a monitor value < 0.1 the median value and will be ignored (no beam ?)')
-                print(np.where(mon <= (mon0 * 0.1)))
-            self.x = np.take(self.x, self.validframes)
-            self.y = np.take(self.y, self.validframes)
-            imgn = np.take(imgn, self.validframes)
-        else:
-            mon = None
-
-        if 'xyrange' in self.params:
-            xmin, xmax, ymin, ymax = self.params['xyrange']
-            print("Restricting scan positions to %f < x < %f and %f < y < %f" % (xmin, xmax, ymin, ymax))
-            idx = np.where((self.x >= xmin) * (self.x <= xmax) * (self.y >= ymin) * (self.y <= ymax))[0]
-            imgn = imgn.take(idx)
-            self.x = self.x.take(idx)
-            self.y = self.y.take(idx)
-
-        if self.params['moduloframe'] is not None:
-            n1, n2 = self.params['moduloframe']
-            idx = np.where(imgn % n1 == n2)[0]
-            imgn = imgn.take(idx)
-            self.x = self.x.take(idx)
-            self.y = self.y.take(idx)
-
-        if self.params['maxframe'] is not None:
-            N = self.params['maxframe']
-            if len(imgn) > N:
-                print("MAXFRAME: only using first %d frames" % (N))
-                imgn = imgn[:N]
-                self.x = self.x[:N]
-                self.y = self.y[:N]
-
         # Load all frames
         sys.stdout.write("Reading %d frames from CXI-HDF5 file: 0" % (len(imgn)))
         sys.stdout.flush()
