@@ -63,6 +63,11 @@ command-line arguments:
                                is equivalent to 'ptychomotors=pix,piz xy=-x,y'
                                [Mandatory]
 
+    xyrange=-2e-6,2e-6,-5e-6,-2e-6: range where the data points will be taken into 
+             account (xmin, xmax, ymin, ymax). All scan positions outside this range are ignored.
+             This must be given in original coordinates, in meters.
+             [default=No spatial restrictions]
+
     monitor=opt2: spec name for the monitor counter. The frames will be normalized by the ratio
                   of the counter value divided by the median value of the counter over the entire
                   scan (so as to remain close to Poisson statistics). A monitor intensity lower 
@@ -73,8 +78,6 @@ command-line arguments:
     kmapfile=detector/kmap/kmap_00000.edf.gz: if images are saved in a multiframe data file 
                                               This superseeds imgname=...
 
-    gpu=Titan: GPU name for OpenCL calculation [default = Titan]
-    
     livescan: if set, this keyword will trigger the analysis of all scans of a given type 
               (by default 'spiralscan') in the supplied spec file. If scan=NN is also given,
               the analysis will start after scan #NN.
@@ -83,7 +86,7 @@ command-line arguments:
 
 params_beamline = {'specfile': None, 'imgcounter': 'auto', 'imgname': None, 'nrj': None, 'detectordistance': None,
                    'pixelsize': 55e-6, 'kmapfile': None, 'monitor': None, 'ptychomotors': None,
-                   'instrument': 'ESRF id01', 'gpu': 'Titan'}
+                   'instrument': 'ESRF id01'}
 
 params = params_generic.copy()
 for k, v in params_beamline.items():
@@ -91,48 +94,20 @@ for k, v in params_beamline.items():
 
 
 class PtychoRunnerScanID01(PtychoRunnerScan):
-    def __init__(self, params, scan):
-        super(PtychoRunnerScanID01, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(PtychoRunnerScanID01, self).__init__(params, scan, timings=timings)
 
-    def load_data(self):
-        self.s = SpecFile(self.params['specfile'])['%d.1' % (self.scan)]
+    def load_scan(self):
+        self.s = SpecFile(self.params['specfile'])['%d.1' % self.scan]
         self.h = self.s.scan_header_dict
-        print('Read scan:', self.h['S'])
-
-        date_string = self.h['D']  # 'Wed Mar 23 14:41:56 2016'
-        date_string = date_string[date_string.find(' ') + 1:]
-        pattern = '%b %d %H:%M:%S %Y'
-        try:
-            lc = locale._setlocale(locale.LC_ALL)
-            locale._setlocale(locale.LC_ALL, 'C')
-            epoch = int(time.mktime(time.strptime(date_string, pattern)))
-            locale._setlocale(locale.LC_ALL, lc)
-            print("SPEC date: %s -> %s " % (date_string, time.strftime("%Y-%m-%dT%H:%M:%S%z", time.localtime(epoch))),
-                  epoch)
-        except ValueError:
-            print(
-                "Could not extract time from spec header, unrecognized format: %s, expected:" % (date_string) + pattern)
-
-        if self.params['nrj'] is None:
-            print("Reading nrj", self.h['UMONO'])
-            self.params['nrj'] = float(self.h['UMONO'].split('mononrj=')[1].split('ke')[0])
-
-        if self.params['detectordistance'] is None and 'UDETCALIB' in self.h:
-            if 'det_distance_CC=' in self.h['UDETCALIB']:
-                # UDETCALIB cen_pix_x=18.347,cen_pix_y=278.971,pixperdeg=445.001,det_distance_CC=1402.175,det_distance_COM=1401.096,timestamp=20170926...
-                self.params['detectordistance'] = float(self.h['UDETCALIB'].split('stance_CC=')[1].split(',')[0])
-                print("Reading detector distance from spec data: det_distance_CC=%6.3fm"
-                      % self.params['detectordistance'])
-
-        if self.params['detectordistance'] is None:
-            raise PtychoRunnerException('Missing argument: no detectordistance given or from scpec header (UDETCALIB)')
+        self.print('Read scan:', self.h['S'])
 
         if self.params['imgcounter'] == 'auto':
             if 'ei2minr' in self.s.labels:
                 self.params['imgcounter'] = 'ei2minr'
             elif 'mpx4inr' in self.s.labels:
                 self.params['imgcounter'] = 'mpx4inr'
-            print("Using image counter: %s" % (self.params['imgcounter']))
+            self.print("Using image counter: %s" % (self.params['imgcounter']))
 
         m = self.params['ptychomotors'].split(',')
         xmot, ymot = m[0:2]
@@ -167,15 +142,29 @@ class PtychoRunnerScanID01(PtychoRunnerScan):
             mon /= mon0
             self.validframes = np.where(mon > 0.1)
             if len(self.validframes) != len(mon):
-                print(
-                    'WARNING: The following frames have a monitor value < 0.1 the median value and will be ignored (no beam ?)')
-                print(np.where(mon <= (mon0 * 0.1)))
+                self.print('WARNING: The following frames have a monitor value < 0.1'
+                           ' the median value and will be ignored (no beam ?)')
+                self.print(np.where(mon <= (mon0 * 0.1)))
             self.x = np.take(self.x, self.validframes)
             self.y = np.take(self.y, self.validframes)
             imgn = np.take(imgn, self.validframes)
         else:
             mon = None
 
+        if 'xyrange' in self.params:
+            xmin, xmax, ymin, ymax = self.params['xyrange']
+            print("Restricting scan positions to %f < x < %f and %f < y < %f" % (xmin, xmax, ymin, ymax))
+            idx = np.where((self.x >= xmin) * (self.x <= xmax) * (self.y >= ymin) * (self.y <= ymax))[0]
+            if len(idx) < 10:
+                raise PtychoRunnerException("Only %d points remaining after applying xyrange"
+                                            "constraint. original range: %5e<x<%5e %5e<y<%5e"
+                                            % (len(idx), self.x.min(), self.x.max(), self.y.min(), self.y.max()))
+            else:
+                print("   ... %d/%d remaining positions" % (len(idx), len(x)))
+            imgn = imgn.take(idx)
+            self.x = self.x.take(idx)
+            self.y = self.y.take(idx)
+
         if self.params['moduloframe'] is not None:
             n1, n2 = self.params['moduloframe']
             idx = np.where(imgn % n1 == n2)[0]
@@ -186,7 +175,7 @@ class PtychoRunnerScanID01(PtychoRunnerScan):
         if self.params['maxframe'] is not None:
             N = self.params['maxframe']
             if len(imgn) > N:
-                print("MAXFRAME: only using first %d frames" % (N))
+                self.print("MAXFRAME: only using first %d frames" % (N))
                 imgn = imgn[:N]
                 self.x = self.x[:N]
                 self.y = self.y[:N]
@@ -201,7 +190,42 @@ class PtychoRunnerScanID01(PtychoRunnerScan):
                 raise PtychoRunnerException("Frame range (%d-%d) does not match expected number of frames (%d)." % (
                     imgn.min(), imgn.max(), len(imgn)))
 
+        self.imgn = imgn
+
+    def load_data(self):
+        self.s = SpecFile(self.params['specfile'])['%d.1' % (self.scan)]
+        self.h = self.s.scan_header_dict
+
+        date_string = self.h['D']  # 'Wed Mar 23 14:41:56 2016'
+        date_string = date_string[date_string.find(' ') + 1:]
+        pattern = '%b %d %H:%M:%S %Y'
+        try:
+            lc = locale._setlocale(locale.LC_ALL)
+            locale._setlocale(locale.LC_ALL, 'C')
+            epoch = int(time.mktime(time.strptime(date_string, pattern)))
+            locale._setlocale(locale.LC_ALL, lc)
+            self.print("SPEC date: %s -> %s " % (date_string, time.strftime("%Y-%m-%dT%H:%M:%S%z",
+                                                                            time.localtime(epoch))), epoch)
+        except ValueError:
+            self.print("Could not extract time from spec header,"
+                       " unrecognized format: %s, expected:" % (date_string) + pattern)
+
+        if self.params['nrj'] is None:
+            self.print("Reading nrj", self.h['UMONO'])
+            self.params['nrj'] = float(self.h['UMONO'].split('mononrj=')[1].split('ke')[0])
+
+        if self.params['detectordistance'] is None and 'UDETCALIB' in self.h:
+            if 'det_distance_CC=' in self.h['UDETCALIB']:
+                # UDETCALIB cen_pix_x=18.347,cen_pix_y=278.971,pixperdeg=445.001,det_distance_CC=1402.175,det_distance_COM=1401.096,timestamp=20170926...
+                self.params['detectordistance'] = float(self.h['UDETCALIB'].split('stance_CC=')[1].split(',')[0])
+                self.print("Reading detector distance from spec data: det_distance_CC=%6.3fm"
+                           % self.params['detectordistance'])
+
+        if self.params['detectordistance'] is None:
+            raise PtychoRunnerException('Missing argument: no detectordistance given or from scpec header (UDETCALIB)')
+
         # Load all frames
+        imgn = self.imgn
         vimg = None
         d0 = 0
         t0 = timeit.default_timer()
@@ -225,19 +249,20 @@ class PtychoRunnerScanID01(PtychoRunnerScan):
                 d0 += frame
                 ii += 1
         else:
-            if self.params['imgname'] == None:
+            if self.params['imgname'] is None:
                 if 'ULIMA_eiger2M' in self.h:
                     imgname = self.h['ULIMA_eiger2M'].strip().split('_eiger2M_')[0] + '_eiger2M_%05d.edf.gz'
-                    print("Using Eiger 2M detector images: %s" % (imgname))
+                    self.print("Using Eiger 2M detector images: %s" % (imgname))
                 else:
                     imgname = self.h['ULIMA_mpx4'].strip().split('_mpx4_')[0] + '_mpx4_%05d.edf.gz'
-                    print("Using Maxipix mpx4 detector images: %s" % (imgname))
+                    self.print("Using Maxipix mpx4 detector images: %s" % (imgname))
             else:
                 imgname = self.params['imgname']
-            sys.stdout.write('Reading frames: ')
+            if self.mpi_master:
+                sys.stdout.write('Reading frames: ')
             ii = 0
             for i in imgn:
-                if (i - imgn[0]) % 20 == 0:
+                if (i - imgn[0]) % 20 == 0 and self.mpi_master:
                     sys.stdout.write('%d ' % (i - imgn[0]))
                     sys.stdout.flush()
                 frame = fabio.open(imgname % i).data
@@ -246,7 +271,7 @@ class PtychoRunnerScanID01(PtychoRunnerScan):
                 vimg[ii] = frame
                 d0 += frame
                 ii += 1
-        print("\n")
+        self.print("\n")
         dt = timeit.default_timer() - t0
         print('Time to read all frames: %4.1fs [%5.2f Mpixel/s]' % (dt, d0.size * len(vimg) / 1e6 / dt))
 
@@ -284,6 +309,8 @@ class PtychoRunnerID01(PtychoRunner):
             else:
                 self.params[k] = v
             return True
+        elif k == 'xyrange':
+            self.params[k] = [float(xy) for xy in v.split(',')]
         return False
 
     def check_params_beamline(self):
