@@ -20,7 +20,7 @@ try:
 except:
     pass
 import fabio
-import h5py
+from ...utils import h5py
 import numpy as np
 
 from .runner import PtychoRunner, PtychoRunnerScan, PtychoRunnerException, params_generic
@@ -95,8 +95,121 @@ for k, v in params_beamline.items():
 
 
 class PtychoRunnerScanID16a(PtychoRunnerScan):
-    def __init__(self, params, scan):
-        super(PtychoRunnerScanID16a, self).__init__(params, scan)
+    def __init__(self, params, scan, timings=None):
+        super(PtychoRunnerScanID16a, self).__init__(params, scan, timings=timings)
+
+    def load_scan(self):
+        m = self.params['ptychomotors'].split(',')
+        if len(m) in [1, 3]:
+            print("reading motors from file:", m[0])
+            # ptycho motor positions from single file, e.g. mot_pos.txt
+            self.x, self.y = np.loadtxt(m[0], unpack=True)
+            # Motor positions are in microns, convert to meters
+            self.x *= 1e-6
+            self.y *= 1e-6
+        else:
+            # ptycho motors from edf file
+            xmot, ymot = m[0:2]
+
+        if self.params['h5data'] is not None:
+            imgn = np.arange(len(self.x), dtype=np.int32)
+            if self.params['moduloframe'] is not None:
+                n1, n2 = self.params['moduloframe']
+                idx = np.where(imgn % n1 == n2)[0]
+                imgn = imgn.take(idx)
+                self.x = self.x.take(idx)
+                self.y = self.y.take(idx)
+
+            if self.params['maxframe'] is not None:
+                N = self.params['maxframe']
+                if len(imgn) > N:
+                    print("MAXFRAME: only using first %d frames" % (N))
+                    imgn = imgn[:N]
+                    self.x = self.x[:N]
+                    self.y = self.y[:N]
+        else:
+            # Load all available frames, as well as motor positions from frame headers (...)
+            i = 0  # Start at 0 or 1 ?
+            if self.params['imgname'].count('%') == 2:
+                if os.path.isfile(self.params['imgname'] % (self.scan, i)) is False:
+                    i = 1
+            elif self.params['imgname'].count('%') == 3:
+                if os.path.isfile(self.params['imgname'] % (self.scan, self.scan, i)) is False:
+                    i = 1
+            else:
+                if os.path.isfile(self.params['imgname'] % i) is False:
+                    i = 1
+
+            vimg = []  # We don't know how many images there are, so start with a list...
+            imgn = []
+            x, y = [], []  # motor positions
+            if self.params['monitor'] is not None:
+                mon = []
+
+            while True:
+                if self.params['moduloframe'] is not None:
+                    n1, n2 = self.params['moduloframe']
+                    if i % n1 != n2:
+                        i += 1
+                        continue
+
+                if i % 20 == 0:
+                    sys.stdout.write('%d ' % i)
+                    sys.stdout.flush()
+
+                imgname = self.params['imgname']
+                if imgname.count('%') == 2:
+                    imgname = imgname % (self.scan, i)
+                elif imgname.count('%') == 3:
+                    imgname = imgname % (self.scan, self.scan, i)
+                else:
+                    imgname = imgname % i
+
+                if os.path.isfile(imgname):
+                    d = fabio.open(imgname)
+                    vimg.append(d.data)
+                    motors = {}
+                    for k, v in zip(d.header['motor_mne'].split(), d.header['motor_pos'].split()):
+                        motors[k] = float(v)
+                    x.append(motors[xmot])  # Motor positions in microns
+                    y.append(motors[ymot])
+                    imgn.append(i)
+                    if self.params['monitor'] is not None:
+                        counters = {}
+                        for k, v in zip(d.header['counter_mne'].split(), d.header['counter_pos'].split()):
+                            counters[k] = float(v)
+                        mon.append(counters[self.params['monitor']])
+                    if self.params['maxframe'] is not None:
+                        if len(vimg) == self.params['maxframe']:
+                            print("MAXFRAME: only using first %d frames" % (len(vimg)))
+                            break
+                    i += 1
+                else:
+                    break
+            self.x = np.array(x) * 1e-6
+            self.y = np.array(y) * 1e-6
+            imgn = np.array(imgn)
+
+        if len(m) >= 3:
+            x, y = self.x, self.y
+            self.x, self.y = eval(m[-2]), eval(m[-1])
+
+        if len(self.x) < 4:
+            raise PtychoRunnerException("Less than 4 scan positions, is this a ptycho scan ?")
+
+        if self.params['monitor'] is not None:
+            mon0 = np.median(mon)
+            mon /= mon0
+            self.validframes = np.where(mon > 0.1)
+            if len(self.validframes) != len(mon):
+                print('WARNING: The following frames have a monitor value < 0.1 x '
+                      'the median value and will be ignored (no beam ?)')
+                print(np.where(mon <= (mon0 * 0.1)))
+            self.x = np.take(self.x, self.validframes)
+            self.y = np.take(self.y, self.validframes)
+            imgn = np.take(imgn, self.validframes)
+
+        self.imgn = imgn
 
     def load_data(self):
         h5meta_filename = self.params['h5meta']
@@ -138,39 +251,12 @@ class PtychoRunnerScanID16a(PtychoRunnerScan):
             self.params['detectordistance'] = float(tmp['focusToDetectorDistance'])  # Seriously ??
             print("Detector distance from h5 metadata: %6.3fm" % self.params['detectordistance'])
 
-        m = self.params['ptychomotors'].split(',')
-        if len(m) in [1, 3]:
-            print("reading motors from file:", m[0])
-            # ptycho motor positions from single file, e.g. mot_pos.txt
-            self.x, self.y = np.loadtxt(m[0], unpack=True)
-            # Motor positions are in microns, convert to meters
-            self.x *= 1e-6
-            self.y *= 1e-6
-        else:
-            # ptycho motors from edf file
-            xmot, ymot = m[0:2]
-
         # read all frames
+        imgn = self.imgn
         t0 = timeit.default_timer()
         sys.stdout.write("Reading frames:")
         sys.stdout.flush()
         if self.params['h5data'] is not None:
-            imgn = np.arange(len(self.x), dtype=np.int32)
-            if self.params['moduloframe'] is not None:
-                n1, n2 = self.params['moduloframe']
-                idx = np.where(imgn % n1 == n2)[0]
-                imgn = imgn.take(idx)
-                self.x = self.x.take(idx)
-                self.y = self.y.take(idx)
-
-            if self.params['maxframe'] is not None:
-                N = self.params['maxframe']
-                if len(imgn) > N:
-                    print("MAXFRAME: only using first %d frames" % (N))
-                    imgn = imgn[:N]
-                    self.x = self.x[:N]
-                    self.y = self.y[:N]
-
             # all images from a single hdf5/nexus file
             # Here assuming lambda detector pseudo-nexus files
             h5data_filename = self.params['h5data']
@@ -178,25 +264,9 @@ class PtychoRunnerScanID16a(PtychoRunnerScan):
                 h5data_filename = h5data_filename % tuple(self.scan for i in range(h5data_filename.count('%')))
             print('h5data filename for scan #%d: %s' % (self.scan, h5data_filename))
             self.h5data = h5py.File(h5data_filename, 'r')
-            imgn = np.arange(len(self.x), dtype=np.int32)
-
-            if self.params['moduloframe'] is not None:
-                n1, n2 = self.params['moduloframe']
-                idx = np.where(imgn % n1 == n2)[0]
-                imgn = imgn.take(idx)
-                self.x = self.x.take(idx)
-                self.y = self.y.take(idx)
-
-            if self.params['maxframe'] is not None:
-                N = self.params['maxframe']
-                if len(imgn) > N:
-                    print("MAXFRAME: only using first %d frames" % (N))
-                    imgn = imgn[:N]
-                    self.x = self.x[:N]
-                    self.y = self.y[:N]
             vimg = self.h5data.get("/entry/instrument/detector/data")[imgn.tolist()]
         else:
-            # Load all available frames, as well as motor positions from frame headers
+            # Load all available frames, as well as motor positions from frame headers (...)
             i = 0  # Start at 0 or 1 ?
             if self.params['imgname'].count('%') == 2:
                 if os.path.isfile(self.params['imgname'] % (self.scan, i)) is False:
@@ -208,18 +278,9 @@ class PtychoRunnerScanID16a(PtychoRunnerScan):
                 if os.path.isfile(self.params['imgname'] % i) is False:
                     i = 1
 
-            vimg = []  # We don't know how many images there are, so start with a list...
-            x, y = [], []  # motor positions
-            if self.params['monitor'] is not None:
-                mon = []
-
-            while True:
-                if self.params['moduloframe'] is not None:
-                    n1, n2 = self.params['moduloframe']
-                    if i % n1 != n2:
-                        i += 1
-                        continue
+            vimg = []
 
+            for i in imgn:
                 if i % 20 == 0:
                     sys.stdout.write('%d ' % i)
                     sys.stdout.flush()
@@ -232,28 +293,8 @@ class PtychoRunnerScanID16a(PtychoRunnerScan):
                 else:
                     imgname = imgname % i
 
-                if os.path.isfile(imgname):
-                    d = fabio.open(imgname)
-                    vimg.append(d.data)
-                    motors = {}
-                    for k, v in zip(d.header['motor_mne'].split(), d.header['motor_pos'].split()):
-                        motors[k] = float(v)
-                    x.append(motors[xmot])  # Motor positions in microns
-                    y.append(motors[ymot])
-                    if self.params['monitor'] is not None:
-                        counters = {}
-                        for k, v in zip(d.header['counter_mne'].split(), d.header['counter_pos'].split()):
-                            counters[k] = float(v)
-                        mon.append(counters[self.params['monitor']])
-                    if self.params['maxframe'] is not None:
-                        if len(vimg) == self.params['maxframe']:
-                            print("MAXFRAME: only using first %d frames" % (len(vimg)))
-                            break
-                    i += 1
-                else:
-                    break
-            self.x = np.array(x) * 1e-6
-            self.y = np.array(y) * 1e-6
+                d = fabio.open(imgname)
+                vimg.append(d.data)
 
         dt = timeit.default_timer() - t0
         print("\n")
@@ -264,27 +305,6 @@ class PtychoRunnerScanID16a(PtychoRunnerScan):
         for i in range(len(vimg)):
             d[i] = vimg[i]
 
-        if len(m) >= 3:
-            x, y = self.x, self.y
-            self.x, self.y = eval(m[-2]), eval(m[-1])
-
-        if len(self.x) < 4:
-            raise PtychoRunnerException("Less than 4 scan positions, is this a ptycho scan ?")
-
-        if self.params['monitor'] is not None:
-            mon0 = np.median(mon)
-            mon /= mon0
-            self.validframes = np.where(mon > 0.1)
-            if len(self.validframes) != len(mon):
-                print(
-                    'WARNING: The following frames have a monitor value < 0.1 the median value and will be ignored (no beam ?)')
-                print(np.where(mon <= (mon0 * 0.1)))
-            self.x = np.take(self.x, self.validframes)
-            self.y = np.take(self.y, self.validframes)
-            # imgn = np.take(imgn, self.validframes)
-        else:
-            mon = None
-
         self.raw_data = d
         self.load_data_post_process()
 
