@@ -17,8 +17,19 @@ import unittest
 import tempfile
 import shutil
 import warnings
-import io
+
+has_mpi = False
+try:
+    from mpi4py import MPI
+    import shutil
+
+    if shutil.which('mpiexec') is not None:
+        has_mpi = True
+except ImportError:
+    pass
+
 from pynx.ptycho.test.test_ptycho import make_ptycho_data, make_ptycho_data_cxi
+from pynx.processing_unit import has_cuda, has_opencl
 
 exclude_cuda = False
 exclude_opencl = False
@@ -48,6 +59,7 @@ class TestPtychoRunner(unittest.TestCase):
 
     @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests skipped")
     @unittest.skipUnless('live_plot' in sys.argv or 'liveplot' in sys.argv, "live plot tests skipped")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
     def test_ptycho_runner_cxi_liveplot_opencl(self):
         my_env = os.environ.copy()
         my_env["PYNX_PU"] = "opencl"
@@ -61,6 +73,7 @@ class TestPtychoRunner(unittest.TestCase):
             self.assertFalse(res, msg=stderr.decode())
 
     @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests skipped")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
     def test_ptycho_runner_cxi_opencl(self):
         my_env = os.environ.copy()
         my_env["PYNX_PU"] = "opencl"
@@ -75,6 +88,7 @@ class TestPtychoRunner(unittest.TestCase):
 
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests skipped")
     @unittest.skipUnless('live_plot' in sys.argv or 'liveplot' in sys.argv, "live plot tests skipped")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
     def test_ptycho_runner_cxi_liveplot_cuda(self):
         my_env = os.environ.copy()
         my_env["PYNX_PU"] = "cuda"
@@ -88,6 +102,7 @@ class TestPtychoRunner(unittest.TestCase):
             self.assertFalse(res, msg=stderr.decode())
 
     @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests skipped")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
     def test_ptycho_runner_cxi_cuda(self):
         my_env = os.environ.copy()
         my_env["PYNX_PU"] = "cuda"
@@ -100,6 +115,48 @@ class TestPtychoRunner(unittest.TestCase):
             res = p.returncode
             self.assertFalse(res, msg=stderr.decode())
 
+    @unittest.skipIf('opencl' in sys.argv or exclude_cuda, "CUDA tests skipped")
+    @unittest.skipIf(has_cuda is False, 'no CUDA support')
+    @unittest.skipIf(has_mpi is False, 'no MPI support')
+    def test_ptycho_runner_cxi_cuda_mpi(self):
+        my_env = os.environ.copy()
+        my_env["PYNX_PU"] = "cuda"
+        path = make_ptycho_data_cxi(dsize=160, nb_frame=40, nb_photons=1e9, dir_name=self.tmp_dir)
+        with subprocess.Popen(['mpiexec', '-n', '2', 'pynx-cxipty.py',
+                               'data=%s' % path, 'probe=gaussian,400e-9x400e-9', 'mpi=split',
+                               'algorithm=analysis,ML**20,DM**20,nbprobe=2,DM**20,nbprobe=1,probe=1',
+                               'verbose=10', 'save=all', 'saveplot'],
+                              stderr=subprocess.PIPE, stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
+            stdout, stderr = p.communicate(timeout=200)
+            res = p.returncode
+            self.assertFalse(res, msg=stderr.decode())
+
+    @unittest.skipIf('cuda' in sys.argv or exclude_opencl, "OpenCL tests skipped")
+    @unittest.skipIf(has_opencl is False, 'no OpenCL support')
+    @unittest.skipIf(has_mpi is False, 'no MPI support')
+    def test_ptycho_runner_cxi_opencl_mpi(self):
+        my_env = os.environ.copy()
+        my_env["PYNX_PU"] = "opencl"
+        path = make_ptycho_data_cxi(dsize=160, nb_frame=40, nb_photons=1e9, dir_name=self.tmp_dir)
+        with subprocess.Popen(['mpiexec', '-n', '2', 'pynx-cxipty.py',
+                               'data=%s' % path, 'probe=gaussian,400e-9x400e-9', 'mpi=split',
+                               'algorithm=analysis,ML**20,DM**20,nbprobe=2,DM**20,nbprobe=1,probe=1',
+                               'verbose=10', 'save=all', 'saveplot'],
+                              stderr=subprocess.PIPE, stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
+            stdout, stderr = p.communicate(timeout=200)
+            res = p.returncode
+            self.assertFalse(res, msg=stderr.decode())
+
+    def test_ptycho_runner_simulation(self):
+        my_env = os.environ.copy()
+        with subprocess.Popen(['pynx-simulationpty.py', 'frame_nb=64', 'frame_size=128',
+                               'algorithm=analysis,ML**20,DM**20,nbprobe=1,probe=1',
+                               'verbose=10', 'save=all', 'saveplot'],
+                              stderr=subprocess.PIPE, stdout=subprocess.PIPE, cwd=self.tmp_dir, env=my_env) as p:
+            stdout, stderr = p.communicate(timeout=200)
+            res = p.returncode
+            self.assertFalse(res, msg=stderr.decode())
+
 
 def suite():
     load_tests = unittest.defaultTestLoader.loadTestsFromTestCase
