@@ -13,7 +13,8 @@ from numpy import arange, sin, exp, pi, newaxis, float32, random, ones, isnan, s
 from pynx import scattering
 
 
-def test_fhkl(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, language="OpenCL", cl_platform="",return_array=False):
+def fhkl(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False,
+         language="OpenCL", cl_platform="", return_array=False):
     # Create array of 3D coordinates, 50x50x50 cells
     x = arange(0, nx, dtype=float32)
     y = arange(0, ny, dtype=float32)[:, newaxis]
@@ -23,25 +24,29 @@ def test_fhkl(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False,
     k = random.uniform(.01, .5, nk)[:, newaxis]
     l = random.uniform(2.01, 2.5, nl)[:, newaxis, newaxis]
     # The actual computation (done twice - first time to compile kernel)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, verbose=verbose, language=language,
-                               cl_platform=cl_platform)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, verbose=verbose, language=language,
-                               cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, verbose=verbose,
+                                       language=language, cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, verbose=verbose,
+                                       language=language, cl_platform=cl_platform)
     # Compare to analytical formula
     fhkl_gold = (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
-                exp(-1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
-    diff = abs(fhkl - fhkl_gold)
+            exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
+                        exp(-1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
+    diff = abs(fhkl_ - fhkl_gold)
     if return_array:
         return diff
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
-    if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  =>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % ("fhkl", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    if tmp > 0.01 or isnan(tmp):
+        s = "FAIL"
+    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
+          ("fhkl", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     return tmp < 0.01
 
 
-def test_fhklo(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, language="OpenCL", cl_platform="",return_array=False):
+def fhklo(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, language="OpenCL",
+          cl_platform="", return_array=False):
     # Create array of 3D coordinates, 50x50x50 cells
     x = arange(0, nx, dtype=float32)
     y = arange(0, ny, dtype=float32)[:, newaxis]
@@ -52,28 +57,28 @@ def test_fhklo(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False
     k = random.uniform(.01, .5, nk)[:, newaxis]
     l = random.uniform(2.01, 2.5, nl)[:, newaxis, newaxis]
     # The actual computation (done twice - first time to compile kernel)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, verbose=verbose, language=language,
-                               cl_platform=cl_platform)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, verbose=verbose, language=language,
-                               cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, verbose=verbose,
+                                       language=language, cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, verbose=verbose,
+                                       language=language, cl_platform=cl_platform)
     # Compare to analytical formula
     fhkl_gold = occ.mean() * (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
-                exp(-1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
-    diff = abs(fhkl - fhkl_gold)
+            exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (
+                        exp(-1j * pi * l * (nz - 1)) * sin(pi * l * nz) / sin(pi * l))
+    diff = abs(fhkl_ - fhkl_gold)
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
     if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print(
-    "%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  =>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % (
-    "fhklo", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % (
+              "fhklo", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     if return_array:
         return diff
     return tmp < 0.01
 
 
-def test_fhklo_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, language="OpenCL",
-                    cl_platform="",return_array=False):
+def fhklo_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False,
+               language="OpenCL", cl_platform="", return_array=False):
     # Create array of 3D coordinates, 50x50x50 cells
     x = arange(0, nx, dtype=float32)
     y = arange(0, ny, dtype=float32)[:, newaxis]
@@ -85,28 +90,29 @@ def test_fhklo_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=
     l = random.uniform(2.01, 2.5, nl)[:, newaxis, newaxis]
     sz_imag = ones(l.shape) * 0.01
     # The actual computation (done twice - first time to compile kernel)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, sz_imag=sz_imag, verbose=verbose,
-                               language=language, cl_platform=cl_platform)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, sz_imag=sz_imag, verbose=verbose,
-                               language=language, cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, sz_imag=sz_imag,
+                                       verbose=verbose, language=language, cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, occ, gpu_name=gpu_name, sz_imag=sz_imag,
+                                       verbose=verbose, language=language, cl_platform=cl_platform)
     # Compare to analytical formula
-    fhkl_gold = occ.mean() * (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (exp(2 * pi * (1j * l + sz_imag) * nz) - 1) / (
-                exp(2 * pi * (1j * l + sz_imag)) - 1)
-    diff = abs(fhkl - fhkl_gold)
+    fhkl_gold = occ.mean() * (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * \
+                (exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * \
+                (exp(2 * pi * (1j * l + sz_imag) * nz) - 1) / (exp(2 * pi * (1j * l + sz_imag)) - 1)
+    diff = abs(fhkl_ - fhkl_gold)
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
     if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print(
-    "%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  =>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % (
-    "fhklo_graz", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
+          ("fhklo_graz", nh * nk * nl // 1000, nx * ny * nz // 1000,
+           nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     if return_array:
         return diff
     return tmp < 0.01
 
 
-def test_fhkl_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, language="OpenCL",
-                   cl_platform="",return_array=False):
+def fhkl_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, language="OpenCL",
+              cl_platform="", return_array=False):
     # Create array of 3D coordinates, 50x50x50 cells
     x = arange(0, nx, dtype=float32)
     y = arange(0, ny, dtype=float32)[:, newaxis]
@@ -117,29 +123,30 @@ def test_fhkl_graz(gpu_name, nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=F
     l = random.uniform(2.01, 2.5, nl)[:, newaxis, newaxis]
     sz_imag = ones(l.shape) * 0.01
     # The actual computation (done twice - first time to compile kernel)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, sz_imag=sz_imag, verbose=verbose, language=language,
-                               cl_platform=cl_platform)
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, sz_imag=sz_imag, verbose=verbose, language=language,
-                               cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, sz_imag=sz_imag,
+                                       verbose=verbose, language=language, cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, gpu_name=gpu_name, sz_imag=sz_imag,
+                                       verbose=verbose, language=language, cl_platform=cl_platform)
     # Compare to analytical formula
-    fhkl_gold = (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * (
-    exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * (exp(2 * pi * (1j * l + sz_imag) * nz) - 1) / (
-                exp(2 * pi * (1j * l + sz_imag)) - 1)
-    diff = abs(fhkl - fhkl_gold)
+    fhkl_gold = (exp(-1j * pi * h * (nx - 1)) * sin(pi * h * nx) / sin(pi * h)) * \
+                (exp(-1j * pi * k * (ny - 1)) * sin(pi * k * ny) / sin(pi * k)) * \
+                (exp(2 * pi * (1j * l + sz_imag) * nz) - 1) / (exp(2 * pi * (1j * l + sz_imag)) - 1)
+    diff = abs(fhkl_ - fhkl_gold)
     tmp = diff.mean() / abs(fhkl_gold).mean()
     s = "PASS"
     if tmp > 0.01 or isnan(tmp): s = "FAIL"
-    print(
-    "%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  =>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" % (
-    "fhkl_graz", nh * nk * nl // 1000, nx * ny * nz // 1000, nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
+    print("%20s: %5d 10^3 reflections, %5d 10^3 atoms, speed=%7.3f 10^9 reflections.atoms/s  "
+          "=>   <|GPU-analytical|> / <|analytical|>=%7.5f, %10s" %
+          ("fhkl_graz", nh * nk * nl // 1000, nx * ny * nz // 1000,
+           nx * ny * nz * nh * nk * nl / dt / 1e9, tmp, s))
     if return_array:
         return diff
     return tmp < 0.01
 
 
-def test_dwba4(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform="",return_array=False):
+def dwba4(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform="",
+          return_array=False):
     from numpy import mgrid, flatnonzero, take, append, sqrt, linspace, float32, abs
-    from scipy.special import erf
     from pynx.scattering import fthomson, gid
 
     # Energy (eV), grazing-incidence angle
@@ -180,7 +187,7 @@ def test_dwba4(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_pla
     ux = x * (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50))
     uy = y * (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50))
     if verbose: print("Simulation with epsilon_xx=epsilon_yy going from %4.2f%% to %4.2f%%" % (
-    0.5, (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50)).max() * 100))
+        0.5, (0.005 + z * 0.001 * (1 + sqrt(x ** 2 + y ** 2) / 50)).max() * 100))
 
     # Compute refraction index for substrate
     si = gid.Scatterer("Si", (0., 0., 0.), 1.0, 1.0 / (8 * pi ** 2), nrj)
@@ -199,50 +206,49 @@ def test_dwba4(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_pla
     l = a * sin(alphaf + alphai) / wavelength
 
     fhklge = gid.fhkl_dwba4(x + ux, y + uy, z, h, k, l=None, occ=occ, alphai=alphai, alphaf=alphaf,
-                           substrate=substrate, wavelength=wavelength,
-                           e_par=0., e_perp=1.0, gpu_name="GTX", language=language, cl_platform=cl_platform)
+                            substrate=substrate, wavelength=wavelength,
+                            e_par=0., e_perp=1.0, gpu_name="GTX", language=language, cl_platform=cl_platform)
 
     fhklsi = gid.fhkl_dwba4(x + ux, y + uy, z, h, k, l=None, occ=1 - occ, alphai=alphai, alphaf=alphaf,
-                           substrate=substrate, wavelength=wavelength,
-                           e_par=1., e_perp=0., gpu_name="GTX", language=language, cl_platform=cl_platform)
+                            substrate=substrate, wavelength=wavelength,
+                            e_par=1., e_perp=0., gpu_name="GTX", language=language, cl_platform=cl_platform)
 
     # Scattering factors
     s = a / sqrt(h ** 2 + k ** 2 + l ** 2)
-    fGe = fthomson.FThomson(s, "Ge")
-    fSi = fthomson.FThomson(s, "Si")
+    fGe = fthomson.f_thomson(s, "Ge")
+    fSi = fthomson.f_thomson(s, "Si")
 
     # Sum scattering
-    fhkl = fhklge * fGe + fhklsi * fSi
+    fhkl_ = fhklge * fGe + fhklsi * fSi
     if show_plot:
         # plot versus H and alpha_f
         from pylab import imshow, cm, xlabel, ylabel, colorbar
-        imshow(abs(fhkl) ** 2, aspect='auto', origin='lower', extent=(h.min(), h.max(), 0, alphaf.max() * 180 / pi),
+        imshow(abs(fhkl_) ** 2, aspect='auto', origin='lower', extent=(h.min(), h.max(), 0, alphaf.max() * 180 / pi),
                cmap=cm.jet)
         xlabel("$H\ (r.l.u.)$", fontsize=18)
         ylabel(r"$\alpha_f\ (^\circ)$", fontsize=18)
         colorbar()
 
     s = "PASS"
-    tmp=abs(fhkl).mean()
+    tmp = abs(fhkl_).mean()
     if tmp < 1 or isnan(tmp): s = "FAIL"
-    print(
-    "%20s: simple DWBA test, 4 paths, using cctbx for refraction index calculations (no strict check)                                         %10s" % (
-    "pynx.gid.FhklDWBA4", s))
+    print("%20s: simple DWBA test, 4 paths, using cctbx for refraction index calculations "
+          "(no strict check)                                         %10s" %
+          ("pynx.gid.FhklDWBA4", s))
     if return_array:
         return fhkl
-    return abs(fhkl).mean() > 1
+    return abs(fhkl_).mean() > 1
 
 
-def test_dwba5(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform="",return_array=False):
-    from numpy import mgrid, flatnonzero, take, append, sqrt, linspace, float32, abs, log10
-    from scipy.special import erf
+def dwba5(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_platform="", return_array=False):
+    from numpy import mgrid, flatnonzero, take, append, linspace, float32, abs, log10
     try:
         from pynx.scattering import fthomson, gid
     except:
         s = "FAIL"
-        print(
-        "%20s: simple DWBA test, 5 paths, using cctbx                                                                  (MISSING cctbx ?) ===>     %10s" % (
-        "pynx.gid.FhklDWBA5", s))
+        print("%20s: simple DWBA test, 5 paths, using cctbx                                      "
+              "                            (MISSING cctbx ?) ===>     %10s" %
+              ("pynx.gid.FhklDWBA5", s))
         return False
     # Simple scattering, small cube above a substrate (larger cube).
     nrj = 10000.
@@ -263,34 +269,34 @@ def test_dwba5(gpu_name, show_plot=True, verbose=True, language="OpenCL", cl_pla
 
     si = gid.Scatterer("Si", (0., 0., 0.), 1.0, 1.0 / (8 * pi ** 2), nrj)
     substrate = gid.Crystal((a, a, a, 90, 90, 90), "Fd3m:1", (si,))
-    fhkl = gid.fhkl_dwba5(x, y, z, h, k, l=None, occ=None, alphai=alphai, alphaf=alphaf,
-                         substrate=substrate, wavelength=wavelength,
-                         e_par=0., e_perp=1.0, gpu_name="GTX", verbose=verbose, language=language,
-                         cl_platform=cl_platform)
+    fhkl_ = gid.fhkl_dwba5(x, y, z, h, k, l=None, occ=None, alphai=alphai, alphaf=alphaf,
+                           substrate=substrate, wavelength=wavelength,
+                           e_par=0., e_perp=1.0, gpu_name="GTX", verbose=verbose, language=language,
+                           cl_platform=cl_platform)
     if show_plot:
         # plot versus H and alpha_f
         from pylab import imshow, cm, xlabel, ylabel, colorbar, figure
         figure(1)
-        imshow(log10(abs(fhkl) ** 2), aspect='auto', origin='lower',
+        imshow(log10(abs(fhkl_) ** 2), aspect='auto', origin='lower',
                extent=(h.min(), h.max(), 0, alphaf.max() * 180 / pi), cmap=cm.jet, vmin=7)
         xlabel(r"$H\ (r.l.u.)$", fontsize=18)
         ylabel(r"$\alpha_f\ (^\circ)$", fontsize=18)
         colorbar()
         figure(2)
-        imshow(log10(abs(fhkl) ** 2), aspect='auto', origin='lower', extent=(h.min(), h.max(), l.min(), l.max()),
+        imshow(log10(abs(fhkl_) ** 2), aspect='auto', origin='lower', extent=(h.min(), h.max(), l.min(), l.max()),
                cmap=cm.jet, vmin=7)
         xlabel(r"$H\ (r.l.u.)$", fontsize=18)
         ylabel(r"$L\ (r.l.u.)$", fontsize=18)
         colorbar()
     s = "PASS"
-    tmp=abs(fhkl).mean()
+    tmp = abs(fhkl_).mean()
     if tmp < 1 or isnan(tmp): s = "FAIL"
-    print(
-    "%20s: simple DWBA test, 5 paths, using cctbx for refraction index calculations (no strict check)                                         %10s" % (
-    "pynx.gid.FhklDWBA5", s))
+    print("%20s: simple DWBA test, 5 paths, using cctbx for refraction index calculations "
+          "(no strict check)                                         %10s" %
+          ("pynx.gid.FhklDWBA5", s))
     if return_array:
         return fhkl
-    return abs(fhkl).mean() > 1
+    return abs(fhkl_).mean() > 1
 
 
 def mrats(nhkl, natoms, gpu_name="GTX", verbose=False, language="OpenCL", cl_platform="", timing=False):
@@ -325,12 +331,13 @@ def mrats(nhkl, natoms, gpu_name="GTX", verbose=False, language="OpenCL", cl_pla
         y += random.normal(0, .02, size=x.shape)
         z += random.normal(0, .02, size=x.shape)
 
-    fhkl, dt = scattering.Fhkl_thread(h, k, l, x, y, z, verbose=False, gpu_name=gpu_name, nbCPUthread=None, language=language,
-                               cl_platform=cl_platform)
+    fhkl_, dt = scattering.Fhkl_thread(h, k, l, x, y, z, verbose=False, gpu_name=gpu_name,
+                                       nbCPUthread=None, language=language, cl_platform=cl_platform)
     MRAtS = nhkl * float(natoms) / dt / 1e6
     fhkl_gold, dt_gold = None, None
-    if verbose: print(
-    "%7d reflections, %8d atoms, dt=%7.5fs , %10.3f MAtoms.reflections/s (%s)" % (nhkl, natoms, dt, MRAtS, gpu_name))
+    if verbose:
+        print("%7d reflections, %8d atoms, dt=%7.5fs , %10.3f MAtoms.reflections/s (%s)" %
+              (nhkl, natoms, dt, MRAtS, gpu_name))
 
     if timing:
         return MRAtS, dt
@@ -349,7 +356,6 @@ def speed(gpu_name, do_plot=False, language="OpenCL", cl_platform="", plot_style
         vnat = []
         vMRAtS = []
         iatoms = 2
-        natoms = 100
         dt = 0
         while dt < 2:
             natoms = int(10 ** iatoms)
@@ -379,7 +385,7 @@ def speed(gpu_name, do_plot=False, language="OpenCL", cl_platform="", plot_style
         show()
 
 
-def test_all(nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, gpu_name=None, language=None, cl_platform=None):
+def run_all(nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, gpu_name=None, language=None, cl_platform=None):
     test_cpu = (language is None or language == "CPU") and (scattering.fhkl.has_cython or scattering.fhkl.has_weave)
     if gpu_name is not None:
         if gpu_name.lower() != 'cpu':
@@ -387,11 +393,11 @@ def test_all(nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, gpu_name=N
 
     if test_cpu:
         print("######## PyNX: testing for device: CPU (C++/SSE)")
-        junk = test_fhkl("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
-        junk = test_fhklo("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
-        junk = test_fhklo_graz("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
-        junk = test_fhkl_graz("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
-        junk = test_dwba5("CPU", show_plot=False, verbose=verbose, language="CPU")
+        junk = fhkl("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
+        junk = fhklo("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
+        junk = fhklo_graz("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
+        junk = fhkl_graz("CPU", nx, ny, nz, nh, nk, nl, verbose=verbose, language="CPU")
+        junk = dwba5("CPU", show_plot=False, verbose=verbose, language="CPU")
 
     test_cuda = scattering.fhkl.drv is not None
     if language is not None:
@@ -406,11 +412,11 @@ def test_all(nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, gpu_name=N
                 if d.lower().find(gpu_name.lower()) < 0:
                     continue
             print("######## PyNX: testing for device (CUDA): ", d)
-            junk = test_fhkl(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
-            junk = test_fhklo(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
-            junk = test_fhklo_graz(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
-            junk = test_fhkl_graz(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
-            junk = test_dwba5(d, show_plot=False, verbose=verbose, language='CUDA')
+            junk = fhkl(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
+            junk = fhklo(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
+            junk = fhklo_graz(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
+            junk = fhkl_graz(d, nx, ny, nz, nh, nk, nl, verbose=verbose, language='CUDA')
+            junk = dwba5(d, show_plot=False, verbose=verbose, language='CUDA')
 
     test_opencl = scattering.fhkl.cl is not None
     if language is not None:
@@ -427,124 +433,116 @@ def test_all(nx=40, ny=40, nz=40, nh=40, nk=40, nl=40, verbose=False, gpu_name=N
                     if d.name.lower().find(gpu_name.lower()) < 0:
                         continue
                 print("######## PyNX: testing for device (OpenCL: platform=%s): %s" % (p.name, d.name))
-                junk = test_fhkl(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL', cl_platform=p.name)
-                junk = test_fhklo(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL',
+                junk = fhkl(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL', cl_platform=p.name)
+                junk = fhklo(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL',
+                             cl_platform=p.name)
+                junk = fhklo_graz(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL',
                                   cl_platform=p.name)
-                junk = test_fhklo_graz(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL',
-                                       cl_platform=p.name)
-                junk = test_fhkl_graz(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL',
-                                      cl_platform=p.name)
-                junk = test_dwba5(d.name, show_plot=False, verbose=verbose, language='OpenCL', cl_platform=p.name)
+                junk = fhkl_graz(d.name, nx, ny, nz, nh, nk, nl, verbose=verbose, language='OpenCL',
+                                 cl_platform=p.name)
+                junk = dwba5(d.name, show_plot=False, verbose=verbose, language='OpenCL', cl_platform=p.name)
 
 
 # TODO dynamically add all tests for all available computing units and platforms
 class TestScattering(unittest.TestCase):
-    @unittest.skipIf(sys.version_info >= (3, 0),"scipy.weave is not compatible with python>=3")
-    def test_Weave(self):
-        msg="Testing for scipy.weave - only available in scipy for python<3. Only needed for CPU calculations, cython can be used instead."
-        try:
-            from scipy import weave
-            import_ok=True
-        except ImportError:
-            import_ok=False
-        self.assertTrue(import_ok,msg=msg)
     def test_Cython(self):
-        msg="Testing for cython - this is only required for CPU calculations if scipy.weave is not available."
+        msg = "Testing for cython"
         try:
             import cython
-            import_ok=True
+            import_ok = True
         except ImportError:
-            import_ok=False
-        self.assertTrue(import_ok,msg=msg)
-    @unittest.skipIf(scattering.fhkl.has_cython==False and scattering.fhkl.has_weave==False,"Neither scipy.weave or cython are available")
+            import_ok = False
+        self.assertTrue(import_ok, msg=msg)
+
+    @unittest.skipIf(scattering.fhkl.has_cython == False, "Cython is not available")
     def test_fhkl_cpu(self):
-        n=20
+        n = 20
 
         # CPU
         if scattering.fhkl.has_cython or scattering.fhkl.has_weave:
             if scattering.fhkl.has_weave:
-                s='scipy.weave'
+                s = 'scipy.weave'
             else:
-                s='cython'
-            msg="Testing fhkl calculations: platform=CPU, language=C/SSE, method=%s"%s
-            res=test_fhkl('CPU', n, n, n, n, n, n, verbose=False, language='CPU',return_array=True)
-            self.assertLess(res.mean(),1,msg=msg)
+                s = 'cython'
+            msg = "Testing fhkl calculations: platform=CPU, language=C/SSE, method=%s" % s
+            res = fhkl('CPU', n, n, n, n, n, n, verbose=False, language='CPU', return_array=True)
+            self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhkl_cuda(self):
-        n=20
+        n = 20
         # CUDA
         scattering.fhkl.drv.init()
         for i in range(scattering.fhkl.drv.Device.count()):
             d = scattering.fhkl.drv.Device(i).name()
-            msg="Testing fhkl calculations: platform=CUDA, device=%s"%d
-            res=test_fhkl(d, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
-            self.assertLess(res.mean(),1,msg=msg)
+            msg = "Testing fhkl calculations: platform=CUDA, device=%s" % d
+            res = fhkl(d, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+            self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhklo_cuda(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhklo calculations: platform=OpenCL, device=%s"%d
-                res=test_fhklo(d.name, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhklo calculations: platform=OpenCL, device=%s" % d
+                res = fhklo(d.name, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhkl_graz_cuda(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhkl_graz calculations: platform=OpenCL, device=%s"%d
-                res=test_fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhkl_graz calculations: platform=OpenCL, device=%s" % d
+                res = fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.drv is None,"PyCUDA is not available")
+    @unittest.skipIf(scattering.fhkl.drv is None, "PyCUDA is not available")
     def test_fhklo_graz_cuda(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhklo_graz calculations: platform=OpenCL, device=%s"%d
-                res=test_fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhklo_graz calculations: platform=OpenCL, device=%s" % d
+                res = fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='CUDA', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    @unittest.skipIf(scattering.fhkl.cl is None, "PyOpenCL is not available")
     def test_fhkl_opencl(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                #if d.type & cl.device_type.GPU is False:
+                # if d.type & cl.device_type.GPU is False:
                 #    continue
-                msg="Testing fhkl calculations: platform=OpenCL, device=%s"%d
-                res=test_fhkl(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhkl calculations: platform=OpenCL, device=%s" % d
+                res = fhkl(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    @unittest.skipIf(scattering.fhkl.cl is None, "PyOpenCL is not available")
     def test_fhklo_opencl(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhklo calculations: platform=OpenCL, device=%s"%d
-                res=test_fhklo(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhklo calculations: platform=OpenCL, device=%s" % d
+                res = fhklo(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    @unittest.skipIf(scattering.fhkl.cl is None, "PyOpenCL is not available")
     def test_fhkl_graz_opencl(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhkl_graz calculations: platform=OpenCL, device=%s"%d
-                res=test_fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhkl_graz calculations: platform=OpenCL, device=%s" % d
+                res = fhkl_graz(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
-    @unittest.skipIf(scattering.fhkl.cl is None,"PyOpenCL is not available")
+    @unittest.skipIf(scattering.fhkl.cl is None, "PyOpenCL is not available")
     def test_fhklo_graz_opencl(self):
-        n=20
+        n = 20
         for p in scattering.fhkl.cl.get_platforms():
             for d in p.get_devices():
-                msg="Testing fhklo_graz calculations: platform=OpenCL, device=%s"%d
-                res=test_fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL',return_array=True)
-                self.assertLess(res.mean(),1,msg=msg)
+                msg = "Testing fhklo_graz calculations: platform=OpenCL, device=%s" % d
+                res = fhklo_graz(d.name, n, n, n, n, n, n, verbose=False, language='OpenCL', return_array=True)
+                self.assertLess(res.mean(), 1, msg=msg)
 
 
 def suite():
