@@ -10,7 +10,8 @@
 from __future__ import division, print_function
 import numpy as np
 from numpy import meshgrid
-import matplotlib.pyplot as plt
+from scipy.fftpack import fftn, ifftn, fftshift, ifftshift
+from pynx.utils.matplotlib import pyplot as plt
 
 __all__ = ['FourierShellCorr', 'FSCPlot', 'HannApod']
 
@@ -92,7 +93,7 @@ class HannApod:
         auxindini = int(np.round(self.outputdim / 2. - self.filterdim / 2.))
         auxindend = int(np.round(self.outputdim / 2. + self.filterdim / 2.))
         hanning_window = self.fract_hanning()
-        out[auxindini:auxindend, auxindini:auxindend] = np.fft.fftshift(hanning_window)
+        out[auxindini:auxindend, auxindini:auxindend] = fftshift(hanning_window)
         # out[auxindini:auxindend, auxindini:auxindend]=np.fft.fftshift(self.fract_hanning(filterdim,unmodsize))
         # return np.fft.fftshift(out)
         return out
@@ -234,11 +235,11 @@ class FourierShellCorr(HannApod):
                 .swapaxes(0, 1).swapaxes(1, 2)
 
         # FSC computation
-        F1 = np.fft.ifftshift(np.fft.fftn(np.fft.fftshift(self.img1 * self.window)))
-        # F1 = np.fft.ifftshift(np.fft.fftn(np.fft.fftshift(self.img1)))
+        F1 = ifftshift(fftn(fftshift(self.img1 * self.window)))
+        # F1 = ifftshift(fftn(fftshift(self.img1)))
         # print(F1.shape)
-        F2 = np.fft.ifftshift(np.fft.fftn(np.fft.fftshift(self.img2 * self.window)))
-        # F2 = np.fft.ifftshift(np.fft.fftn(np.fft.fftshift(self.img2)))
+        F2 = ifftshift(fftn(fftshift(self.img2 * self.window)))
+        # F2 = ifftshift(fftn(fftshift(self.img2)))
         C, C1, C2, npts = [[], [], [], []]
         index = self.ringthickness()
         f, fnyquist = self.nyquist()
