@@ -9,20 +9,29 @@
 #         Ondrej Mandula
 from __future__ import division
 
-__all__ = ['phase_tilt', 'grad_phase', 'phase_diff', 'minimize_grad_phase', 'shift_phase_zero', 'unwrap_phase']
+__all__ = ['phase_tilt', 'grad_phase', 'phase_diff', 'minimize_grad_phase', 'shift_phase_zero', 'unwrap_phase',
+           'remove_phase_ramp']
 
+import warnings
 import numpy as np
 from numpy import pi
+from scipy.fftpack import fftn, ifftn, fftshift
 from scipy.optimize import minimize
 from scipy.signal import medfilt2d
 from scipy.integrate import cumtrapz
+from scipy.ndimage import fourier_shift
 from skimage.restoration import unwrap_phase as sk_unwrap_phase
+from skimage.feature import register_translation
+from skimage import __version__ as skimage_version
+from packaging.version import parse as version_parse
+from .array import rebin
 
 
 def phase_tilt(im, grad=(0, 0), offset=0):
     """
     Multiplies the phase of the complex image by the gradient and adds phase offset.
-    
+    DEPRECATED. scipy.ndimage.fourier_shift should be used instead
+
     Args:
         im: input image
         grad: gradient of the phase as a multiple of 2*pi across the image
@@ -38,7 +47,7 @@ def phase_tilt(im, grad=(0, 0), offset=0):
 def grad_phase(p, im):
     # oPhaseMasked = np.ma.masked_array(np.angle(im*np.exp(1j*2*np.pi*(xx*p[0]/im.shape[0]+yy*p[1]/im.shape[1]))),-mask)
     # oPhaseMasked = np.ma.masked_array(np.angle(im*np.exp(1j*(xx*p[0]+yy*p[1]))),-mask)
-    oPhaseMasked = np.angle(phaseTilt(im, p))
+    oPhaseMasked = np.angle(phase_tilt(im, p))
     dx, dy = np.gradient(oPhaseMasked)
     #    return sum(abs(dx)+abs(dy))
     return np.median(abs(dx) + abs(dy))
@@ -52,20 +61,19 @@ def phase_diff(p, im, x, y):
     return (abs(gx) ** 2 + abs(gy) ** 2).sum()
 
 
-def minimize_grad_phase(im, mask_thr=0.3, center_phase=None, global_min=False, mask=None):
+def minimize_grad_phase(im, mask_thr=0.3, center_phase=None, global_min=False, mask=None, rebin_f=None):
     """
     Minimises the gradient in the phase of the input image im.
-    TODO: faster, directly compute gradient instead of relying on scipy.optimize.minimize...
-    
+
     Args:
         im: the input complex 2D image for which the phase gradient needs to be minimized
         mask_thr: only pixel amplitudes above mask_thr*abs(im).max() will be taken into account
-        init_grad: initial gradient (deprecated, ignored in current implementation)
         center_phase: after minimizing the gradient, center the phase around this value (in radians). pi/3 gives a nice contrast
         global_min: if True after a quick local minimization of the gradient, the phase differences will be minimized over the whole object.
                     Use this for flat-phased objects
         mask: a 2d mask array (True = masked points) giving the pixels which should be excluded from the optimization.
               This is cumulative with mask_thr
+        rebin_f: an integer>1 can be given to compute the phase gradient on a rebinned array for faster evaluation.
     Returns:
         a tuple with (phase corrected image, correction array, mask, linear coefficients)
     """
@@ -78,6 +86,15 @@ def minimize_grad_phase(im, mask_thr=0.3, center_phase=None, global_min=False, m
         mask = (abs(im) / abs(im).max()) < mask_thr
         if (~mask).sum() < 1000:
             mask = np.zeros(im.shape, dtype=np.bool)
+    mask = mask.astype(np.bool)
+
+    if rebin_f is not None:
+        im0 = im
+        mask0 = mask
+        im = rebin(im, rebin_f, normalize=False)
+        m = rebin((mask == 0).astype(np.int8), rebin_f, normalize=False)
+        im /= m + (m == 0)
+        mask = m == 0
 
     ny, nx = im.shape
     gradient_step = 10
@@ -114,7 +131,11 @@ def minimize_grad_phase(im, mask_thr=0.3, center_phase=None, global_min=False, m
         res = minimize(phase_diff, (0, 0), args=(im2m, x, y), method='Powell', options={'xtol': 1e-18, 'disp': False})
         grad = [grad[0] + res['x'][0], grad[1] + res['x'][1]]
 
+    if rebin_f is not None:
+        im = im0
+        mask = mask0
     imnew = phase_tilt(im, grad)
+
     corr = phase_tilt(np.ones(im.shape), grad)
     if center_phase is not None:
         imm = imnew[mask == 0]
@@ -127,6 +148,72 @@ def minimize_grad_phase(im, mask_thr=0.3, center_phase=None, global_min=False, m
     return imnew, corr, mask, grad
 
 
+def remove_phase_ramp(d: np.ndarray, mask_thr=None, center_phase=None, mask=None, niter=2):
+    """
+     Removes a linear ramp in the phase of the input array (2D or 3D).
+     This uses scikit-image register_translation in Fourier space for a fast evaluation, by calculating the shift
+     of FT(d) compared to FT(abs(d)). 3D correction requires scikit-image>=0.15.0.
+
+     WARNING: This works efficiently on a pure phase ramp, with some noise, but cannot be used in general - e.g. on an
+     object with a phase vortex, FT(d) and FT(abs(d)) can be quite different and cannot easily be correlated.
+
+    :param d: the data array for which the phase ramp must be removed.
+    :param mask_thr: only pixel amplitudes above mask_thr*abs(im).max() will be taken into account
+    :param center_phase: after minimizing the gradient, center the phase around this value (in radians).
+                         pi/3 gives a nice contrast.
+    :param mask: a mask array (True = masked points) giving the pixels which should be excluded from the optimization.
+                 This is cumulative with mask_thr. Masked pixels are corrected, but not used to evaluate the phase
+                 gradient.
+    :param niter: number of iterations to perform to remove ramp d (default=2)
+    :return: a tuple with (phase corrected array, linear coefficients). The linear
+             coefficients correspond to the total amplitude of the ramp along the given axis, in radians.
+    """
+    dm = d
+    if mask is not None:
+        dm = dm * (mask > 0)
+    if mask_thr is not None:
+        dm *= abs(dm) >= mask_thr
+    upsample_factor = 20
+
+    if dm.ndim == 3:
+        assert version_parse(skimage_version) >= version_parse('0.15'), \
+            'scikit-image>=0.15.0 is needed for 3D remove_phase_ramp'
+
+    dmaf = fftshift(fftn(fftshift(np.abs(dm))))
+    coeffs = np.zeros(dm.ndim, dtype=np.float32)
+
+    for i in range(niter):
+        phase_power = 20 ** (i + 1)
+        dmf = fftshift(fftn(fftshift(np.abs(dm) * np.exp(1j * np.angle(dm) * phase_power))))
+        dmf = dmf.astype(np.complex64)
+        # Find phase ramp using register_translation in Fourier space
+        s, err, dphi = register_translation(dmaf, dmf, upsample_factor=upsample_factor)
+        s = -np.array(s) / phase_power
+        coeffs += s
+        # Correct array using fourier_shift
+        dm = fftshift(fourier_shift(fftshift(dm), s))
+        # Same correction without fourier_shift (in 3D):
+        # nz, ny, nx = dm.shape
+        # z, y, x = np.meshgrid(np.arange(nz) / nz - 0.5, np.arange(ny) / ny - 0.5, np.arange(nx) / nx - 0.5,
+        #                       indexing='ij')
+        # phase_corr = [(v * 2 * np.pi) for v in s]
+        # dm = dm * np.exp(1j * (z * phase_corr[0] + y * phase_corr[1] + x * phase_corr[2] + dphi1))
+
+    # Final correction
+    dcorr = fftshift(fourier_shift(fftshift(d), coeffs))
+
+    if center_phase is not None:
+        dm = d
+        if mask is not None:
+            dm = dm * (mask > 0)
+        if mask_thr is not None:
+            dm *= abs(dm) >= mask_thr
+        # Is that the most efficient phase centering ?
+        dcorr *= np.exp(-1j * np.angle(dm.sum()))
+
+    return dcorr.astype(d.dtype), coeffs * 2 * np.pi
+
+
 def shift_phase_zero(obj, percent=5, mask=None, origin=0):
     """
     Shift the phase of the given object, so that the phase range begins at 0 radians, and hopefully avoid wrapped
@@ -204,9 +291,3 @@ def unwrap_phase(d: np.array, method='skimage', medfilt_n=3):
     a = np.angle(d)
     a += np.round((ph - a) / (2 * np.pi)) * (2 * np.pi)
     return a
-
-
-# Deprecated names
-minimizeGradPhase = minimize_grad_phase
-gradPhase = grad_phase
-phaseTilt = phase_tilt
