@@ -178,7 +178,7 @@ class CLOperatorWavefront(OperatorWavefront):
             src = obj._cl_d
         else:
             src = obj._cl_d_view[i_source]
-        if i_dest is 0:
+        if i_dest == 0:
             obj._cl_d = cl.array.empty_like(src)
             dest = obj._cl_d
         else:
@@ -339,7 +339,7 @@ class FreePU(CLOperatorWavefront):
         for o in dir(w):
             if isinstance(w.__getattribute__(o), pyopencl.array.Array):
                 w.__setattr__(o, None)
-        self.processing_unit.gpyfft_plan.data = None
+        self.processing_unit.cl_fft_free_plans()
         self.view_purge(w, None)
         return w
 
@@ -521,8 +521,8 @@ class FT(CLOperatorWavefront):
         :param w: the Wavefront object this operator applies to
         :return: the updated Wavefront object
         """
-        self.processing_unit.cl_fft_set_plan(w._cl_d)
-        for e in self.processing_unit.gpyfft_plan.enqueue(forward=True):
+        plan = self.processing_unit.cl_fft_get_plan(w._cl_d, shuffle_axes=True)
+        for e in plan.enqueue(forward=True):
             e.wait()
         self.processing_unit.cl_scale(w._cl_d, np.float32(1 / np.sqrt(w._d.size)))
         return w
@@ -539,8 +539,8 @@ class IFT(CLOperatorWavefront):
         :param w: the Wavefront object this operator applies to
         :return: the updated Wavefront object
         """
-        self.processing_unit.cl_fft_set_plan(w._cl_d)
-        for e in self.processing_unit.gpyfft_plan.enqueue(forward=False):
+        plan = self.processing_unit.cl_fft_get_plan(w._cl_d, shuffle_axes=True)
+        for e in plan.enqueue(forward=False):
             e.wait()
         self.processing_unit.cl_scale(w._cl_d, np.float32(np.sqrt(w._d.size)))
         return w
