@@ -99,7 +99,7 @@ class CPUOperatorWavefront(OperatorWavefront):
             src = obj._obj
         else:
             src = obj._cpu_d_view[i_source]
-        if i_dest is 0:
+        if i_dest == 0:
             obj._cpu_obj = np.empty_like(src)
             dest = obj._obj
         else:
@@ -238,8 +238,8 @@ class Imshow(CPUOperatorWavefront):
         else:
             plt.title(self.title)
         try:
-            # Not supported on all backends (e.g. nbagg)
             plt.draw()
+            plt.gcf().canvas.draw()
             plt.pause(.001)
         except:
             pass
@@ -826,11 +826,12 @@ class BackPropagatePaganin(CPUOperatorWavefront):
         w = FT() * w
         mu = np.float32(4 * np.pi * self.beta / w.wavelength)
         p = w.pixel_size
-        dk = np.float32(2 * np.pi / (w._d.shape[-1] * p))
+        dkx = np.float32(2 * np.pi / (w._d.shape[-1] * p))
+        dky = np.float32(2 * np.pi / (w._d.shape[-2] * p))
 
         ny, nx = w._d.shape[-2:]
-        kx = fftshift(np.arange(-nx // 2, nx // 2)) * dk
-        ky = fftshift(np.arange(-ny // 2, ny // 2)) * dk
+        kx = fftshift(np.arange(-nx // 2, nx // 2)) * dkx
+        ky = fftshift(np.arange(-ny // 2, ny // 2)) * dky
         ky, kx = np.meshgrid(ky, kx, indexing='ij')
         dz = self.dz
         delta = self.delta
