@@ -192,7 +192,7 @@ class CUOperatorWavefront(OperatorWavefront):
             src = obj._cu_d
         else:
             src = obj._cu_d_view[i_source]
-        if i_dest is 0:
+        if i_dest == 0:
             obj._cu_d = cua.empty_like(src)
             dest = obj._cu_d
         else:
@@ -327,6 +327,8 @@ class FreePU(CUOperatorWavefront):
     """
 
     def op(self, w):
+        self.processing_unit.finish()
+        self.processing_unit.cu_fft_free_plans()
         w.get()  # This will copy the data to the host memory if necessary
         for o in dir(w):
             if isinstance(w.__getattribute__(o), cua.GPUArray):
@@ -480,8 +482,8 @@ class FT(CUOperatorWavefront):
     """
 
     def op(self, w):
-        self.processing_unit.cu_fft_set_plan(w._cu_d, batch=True)
-        cu_fft.fft(w._cu_d, w._cu_d, self.processing_unit.cufft_plan, scale=False)
+        plan = self.processing_unit.cu_fft_get_plan(w._cu_d, batch=True)
+        cu_fft.fft(w._cu_d, w._cu_d, plan, scale=False)
         w._cu_d *= np.float32(1 / np.sqrt(w._d.size))
         return w
 
@@ -493,8 +495,8 @@ class IFT(CUOperatorWavefront):
     """
 
     def op(self, w):
-        self.processing_unit.cu_fft_set_plan(w._cu_d, batch=True)
-        cu_fft.ifft(w._cu_d, w._cu_d, self.processing_unit.cufft_plan, scale=False)
+        plan = self.processing_unit.cu_fft_get_plan(w._cu_d, batch=True)
+        cu_fft.ifft(w._cu_d, w._cu_d, plan, scale=False)
         w._cu_d *= np.float32(1 / np.sqrt(w._d.size))
         return w
 
